@@ -7,7 +7,11 @@
 #include <stdlib.h>
 #include <stdint.h>
 
+#if !defined(HAVE_GLES2)
 #include "GLee.h"
+#else
+#include <GLES2/gl2.h>
+#endif
 
 #include "allfiles.h"
 
@@ -62,7 +66,7 @@ extern int cameraX, cameraY;
 extern float cameraZoom;
 
 void nosnapshot () {
-	glDeleteTextures (1, &snapshotTextureName);
+	deleteTextures (1, &snapshotTextureName);
 	snapshotTextureName = 0;
 }
 
@@ -80,7 +84,7 @@ bool snapshot () {
 	nosnapshot ();
 	if (! freeze ()) return false;
 
-	glEnable (GL_TEXTURE_2D);
+
 	setPixelCoords (true);
 
 	glGenTextures (1, &snapshotTextureName);
@@ -98,7 +102,7 @@ bool snapshot () {
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-	glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
+	texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0, snapshotTextureName);
 
 	// Render scene
 	glDepthMask (GL_TRUE);
@@ -115,8 +119,7 @@ bool snapshot () {
 	drawStatusBar ();
 
 	// Copy Our ViewPort To The Texture
-	glBindTexture(GL_TEXTURE_2D, snapshotTextureName);
-	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, viewportOffsetX, viewportOffsetY, winWidth, winHeight);
+	copyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, viewportOffsetX, viewportOffsetY, winWidth, winHeight, snapshotTextureName);
 
 	setPixelCoords (false);
 
@@ -171,7 +174,7 @@ bool restoreSnapshot (FILE * fp) {
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-	glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, snapshotTexture);
+	texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, snapshotTexture, snapshotTextureName);
 
 	delete snapshotTexture;
 	snapshotTexture = NULL;
@@ -181,13 +184,13 @@ bool restoreSnapshot (FILE * fp) {
 
 
 void killBackDrop () {
-	glDeleteTextures (1, &backdropTextureName);
+	deleteTextures (1, &backdropTextureName);
 	backdropTextureName = 0;
 	backdropExists = false;
 }
 
 void killLightMap () {
-	glDeleteTextures (1, &lightMap.name);
+	deleteTextures (1, &lightMap.name);
 	lightMap.name = 0;
 	if (lightMap.data) {
 		delete lightMap.data;
@@ -203,7 +206,7 @@ void killParallax () {
 		parallaxStuff = k -> next;
 
 		// Now kill the image
-		glDeleteTextures (1, &k -> textureName);
+		deleteTextures (1, &k -> textureName);
 		delete k -> texture;
 		delete k;
 		k = NULL;
@@ -232,6 +235,7 @@ bool reserveBackdrop () {
 		backdropTexH = ((double)sceneHeight) / picHeight;
 	}
 	backdropTexture = new GLubyte [picWidth*picHeight*4];
+	if (! checkNew (backdropTexture)) return false;
 
 	if (! backdropTextureName) glGenTextures (1, &backdropTextureName);
 	glBindTexture (GL_TEXTURE_2D, backdropTextureName);
@@ -245,7 +249,7 @@ bool reserveBackdrop () {
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 	}
 
-	glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, backdropTexture);
+	texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, backdropTexture, backdropTextureName);
 
 	return true;
 }
@@ -314,34 +318,30 @@ void blankScreen (int x1, int y1, int x2, int y2) {
 			int h = (picHeight-yoffset < viewportHeight) ? picHeight-yoffset : viewportHeight;
 
 			// Render the scene
-			glColor3ub(redValue(currentBlankColour), greenValue(currentBlankColour), blueValue(currentBlankColour));
 
-			glDisable (GL_TEXTURE_2D);
+		
 
 			const GLfloat vertices[] = { 
 				-10.325f, -1.325f, 0.0f, 
 				w+1.325f, -1.325f, 0.0f, 
-				w+1.325f, h+1.325f, 0.0f, 
-				-10.325f, h+1.325f, 0.0f
+				-10.325f, h+1.325f, 0.0f,
+				w+1.325f, h+1.325f, 0.0f 
 			};
 
-			glEnableClientState(GL_VERTEX_ARRAY);
-
-			glVertexPointer(3, GL_FLOAT, 0, vertices);
-
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_VERTEX_ARRAY);
+			glUseProgram(shader.color);
+			setPMVMatrix(shader.color);
+			setPrimaryColor(redValue(currentBlankColour)/255.0f, greenValue(currentBlankColour)/255.0f, blueValue(currentBlankColour)/255.0f, 1.0f);
+			drawQuad(shader.color, vertices, 0);
+			glUseProgram(0);
 
 			// Copy Our ViewPort To The Texture
-			glBindTexture(GL_TEXTURE_2D, backdropTextureName);
-			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, x1+xoffset, y1+yoffset, viewportOffsetX, viewportOffsetY, w, h);
+			copyTexSubImage2D(GL_TEXTURE_2D, 0, x1+xoffset, y1+yoffset, viewportOffsetX, viewportOffsetY, w, h, backdropTextureName);
 
 			yoffset += viewportHeight;
 		}
 		xoffset += viewportWidth;
 	}
-	glEnable (GL_TEXTURE_2D);
+
 	setPixelCoords (false);
 }
 
@@ -356,8 +356,8 @@ void hardScroll (int distance) {
 	const GLfloat backdropTexCoords[] = { 
 		0.0f, 0.0f,
 		backdropTexW, 0.0f,
-		backdropTexW, backdropTexH, 
-		0.0f, backdropTexH
+		0.0f, backdropTexH,
+		backdropTexW, backdropTexH
 	}; 
 
 	setPixelCoords (true);
@@ -377,27 +377,21 @@ void hardScroll (int distance) {
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 
-			const GLint vertices[] = { 
-				-xoffset, -distance-yoffset, 0, 
-				sceneWidth-xoffset, -distance-yoffset, 0, 
-				sceneWidth-xoffset, sceneHeight-distance-yoffset, 0, 
-				-xoffset, sceneHeight-distance-yoffset, 0
+			const GLfloat vertices[] = { 
+				(GLfloat)-xoffset, (GLfloat)-distance-yoffset, 0., 
+				(GLfloat)sceneWidth-xoffset, (GLfloat)-distance-yoffset, 0., 
+				(GLfloat)-xoffset, (GLfloat)sceneHeight-distance-yoffset, 0.,
+				(GLfloat)sceneWidth-xoffset, (GLfloat)sceneHeight-distance-yoffset, 0. 
 			};
 
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+			glUseProgram(shader.texture);
+			setPMVMatrix(shader.texture);
 
-			glVertexPointer(3, GL_INT, 0, vertices);
-			glTexCoordPointer(2, GL_FLOAT, 0, backdropTexCoords);
-
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
+			drawQuad(shader.texture, vertices, 1, backdropTexCoords);
+			glUseProgram(0);
 
 			// Copy Our ViewPort To The Texture
-			glBindTexture(GL_TEXTURE_2D, backdropTextureName);
-			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, xoffset, yoffset, viewportOffsetX, viewportOffsetY, w, h);
+			copyTexSubImage2D(GL_TEXTURE_2D, 0, xoffset, yoffset, viewportOffsetX, viewportOffsetY, w, h, backdropTextureName);
 			
 			yoffset += viewportHeight;
 		}
@@ -426,55 +420,48 @@ void darkScreen () {
 			int h = (sceneHeight-yoffset < viewportHeight) ? sceneHeight-yoffset : viewportHeight;
 
 			// Render the scene - first the old backdrop
-			glEnable (GL_TEXTURE_2D);
-			glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+		
+			//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 			glBindTexture (GL_TEXTURE_2D, backdropTextureName);
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 
-			const GLint vertices[] = { 
-				-xoffset, -yoffset, 0, 
-				sceneWidth-xoffset, -yoffset, 0, 
-				sceneWidth-xoffset, sceneHeight-yoffset, 0, 
-				-xoffset, sceneHeight-yoffset, 0
+			const GLfloat vertices[] = { 
+				(GLfloat)-xoffset, (GLfloat)-yoffset, 0., 
+				(GLfloat)sceneWidth-xoffset, (GLfloat)-yoffset, 0., 
+				(GLfloat)-xoffset, (GLfloat)sceneHeight-yoffset, 0.,
+				(GLfloat)sceneWidth-xoffset, (GLfloat)sceneHeight-yoffset, 0.
 			};
 
 			const GLfloat texCoords[] = { 
 				0.0f, 0.0f,
 				backdropTexW, 0.0f,
-				backdropTexW, backdropTexH, 
-				0.0f, backdropTexH
+				0.0f, backdropTexH,
+				backdropTexW, backdropTexH
 			}; 
-	
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 
-			glVertexPointer(3, GL_INT, 0, vertices);
-			glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
+			glUseProgram(shader.texture);
+			setPMVMatrix(shader.texture);
 
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
+			drawQuad(shader.texture, vertices, 1, texCoords);
 
 			// Then the darkness
-			glDisable (GL_TEXTURE_2D);
-			glColor4ub (0, 0, 0, 127);
+		
 			glColorMask (GL_TRUE, GL_TRUE, GL_TRUE, GL_FALSE);
 
 			glEnable(GL_BLEND);
 
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glVertexPointer(3, GL_INT, 0, vertices);
-			glDrawArrays(GL_QUADS, 0, 4);
-			glDisableClientState(GL_VERTEX_ARRAY);
+			glUseProgram(shader.color);
+			setPMVMatrix(shader.color);
+			setPrimaryColor(0.0f, 0.0f, 0.0f, 0.5f);
+			drawQuad(shader.color, vertices, 0);
+			glUseProgram(0);
 
 			glDisable(GL_BLEND);
 			glColorMask (GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
 
 			// Copy Our ViewPort To The Texture
-			glBindTexture(GL_TEXTURE_2D, backdropTextureName);
-			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, xoffset, yoffset, viewportOffsetX, viewportOffsetY, w, h);
+			copyTexSubImage2D(GL_TEXTURE_2D, 0, xoffset, yoffset, viewportOffsetX, viewportOffsetY, w, h, backdropTextureName);
 
 			yoffset += h;
 			glClear(GL_COLOR_BUFFER_BIT);
@@ -482,7 +469,8 @@ void darkScreen () {
 		}
 		xoffset += w;
 	}
-	glEnable (GL_TEXTURE_2D);
+
+
 	setPixelCoords (false);
 }
 
@@ -498,15 +486,21 @@ inline int sortOutPCamera (int cX, int fX, int sceneMax, int boxMax) {
 
 void drawBackDrop () {
 
-	glColor4f(1.0, 1.0, 1.0, 1.0);
-	glEnable (GL_TEXTURE_2D);
-	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+	setPrimaryColor(1.0, 1.0, 1.0, 1.0);
+
+	//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 	glEnable(GL_BLEND);
 
+	glUseProgram(shader.smartScaler);
+	GLuint uniform = glGetUniformLocation(shader.smartScaler, "useLightTexture");
+	if (uniform >= 0) glUniform1i(uniform, 0);
+
+	setPMVMatrix(shader.smartScaler);
+
 	if (gameSettings.antiAlias == 1) {
-		glUseProgram(shader.smartScaler);
-		GLuint uniform = glGetUniformLocation(shader.smartScaler, "useLightTexture");
-		if (uniform >= 0) glUniform1i(uniform, 0);
+		glUniform1i(glGetUniformLocation(shader.smartScaler, "antialias"), 1);
+	} else {
+		glUniform1i(glGetUniformLocation(shader.smartScaler, "antialias"), 0);
 	}
 
 	if (parallaxStuff) {
@@ -535,27 +529,17 @@ void drawBackDrop () {
 			const GLfloat vertices[] = { 
 				(GLfloat)-ps -> cameraX, (GLfloat)-ps -> cameraY, 0.1f, 
 				w -ps -> cameraX, (GLfloat)-ps -> cameraY, 0.1f, 
-				w -ps -> cameraX, h -ps -> cameraY, 0.1f, 
-				(GLfloat)-ps -> cameraX, h -ps -> cameraY, 0.1f
+				(GLfloat)-ps -> cameraX, h -ps -> cameraY, 0.1f,
+				w -ps -> cameraX, h -ps -> cameraY, 0.1f
 			};
 
 			const GLfloat texCoords[] = { 
 				0.0f, 0.0f,
 				texw, 0.0f,
-				texw, texh, 
-				0.0f, texh
+				0.0f, texh,
+				texw, texh
 			}; 
-	
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-			glVertexPointer(3, GL_FLOAT, 0, vertices);
-			glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
-
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
+			drawQuad(shader.smartScaler, vertices, 1, texCoords);
 
 			ps = ps -> prev;
 		}
@@ -566,27 +550,18 @@ void drawBackDrop () {
 	const GLfloat backdropTexCoords[] = { 
 		0.0f, 0.0f,
 		backdropTexW, 0.0f,
-		backdropTexW, backdropTexH, 
-		0.0f, backdropTexH
+		0.0f, backdropTexH,
+		backdropTexW, backdropTexH
 	}; 
 
-	const GLint vertices[] = { 
-		-cameraX, -cameraY, 0, 
-		sceneWidth-cameraX, -cameraY, 0, 
-		sceneWidth-cameraX, sceneHeight-cameraY, 0, 
-		-cameraX, sceneHeight-cameraY, 0
+	const GLfloat vertices[] = { 
+		(GLfloat)-cameraX, (GLfloat)-cameraY, 0., 
+		(GLfloat)sceneWidth-cameraX, (GLfloat)-cameraY, 0., 
+		(GLfloat)-cameraX, (GLfloat)sceneHeight-cameraY, 0.,
+		(GLfloat)sceneWidth-cameraX, (GLfloat)sceneHeight-cameraY, 0.
 	};
 
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-	glVertexPointer(3, GL_INT, 0, vertices);
-	glTexCoordPointer(2, GL_FLOAT, 0, backdropTexCoords);
-
-	glDrawArrays(GL_QUADS, 0, 4);
-
-	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-	glDisableClientState(GL_VERTEX_ARRAY);
+	drawQuad(shader.smartScaler, vertices, 1, backdropTexCoords);
 
 	glDisable(GL_BLEND);
 
@@ -737,7 +712,7 @@ bool loadLightMap (int v) {
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-	glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, newPicWidth, newPicHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, lightMap.data);
+	texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, newPicWidth, newPicHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, lightMap.data, lightMap.name);
 
 	finishAccess ();
 	setResourceForFatal (-1);
@@ -772,9 +747,9 @@ void reloadParallaxTextures () {
 		}
 
 		if (! NPOT_textures) {
-			glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, getNextPOT(nP->width), getNextPOT(nP->height), 0, GL_RGBA, GL_UNSIGNED_BYTE, nP->texture);
+			texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, getNextPOT(nP->width), getNextPOT(nP->height), 0, GL_RGBA, GL_UNSIGNED_BYTE, nP->texture, nP->textureName);
 		} else {
-			glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, nP->width, nP->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, nP->texture);
+			texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, nP->width, nP->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, nP->texture, nP->textureName);
 		}
 		nP = nP->next;
 	}
@@ -894,7 +869,6 @@ bool loadParallax (unsigned short v, unsigned short fracX, unsigned short fracY)
 	}
 
 	nP -> texture = new GLubyte [picHeight * picWidth * 4];
-
 	if (! checkNew (nP -> texture)) return false;
 
 	if (fileIsPNG) {
@@ -959,7 +933,7 @@ bool loadParallax (unsigned short v, unsigned short fracX, unsigned short fracY)
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 	}
 
-	glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, nP->texture);
+	texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, nP->texture, nP->textureName);
 
 	finishAccess ();
 	setResourceForFatal (-1);
@@ -1114,10 +1088,10 @@ bool loadHSI (FILE * fp, int x, int y, bool reserve) {
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 	}
-	glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, backdropTexture);
+	texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, backdropTexture, tmpTex);
 
-	glEnable (GL_TEXTURE_2D);
-	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+
+	//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 
 
 	float btx1;
@@ -1139,8 +1113,8 @@ bool loadHSI (FILE * fp, int x, int y, bool reserve) {
 	const GLfloat btexCoords[] = { 
 		btx1, bty1,
 		btx2, bty1,
-		btx2, bty2, 
-		btx1, bty2
+		btx1, bty2,
+		btx2, bty2 
 	}; 
 
 	setPixelCoords (true);
@@ -1155,87 +1129,65 @@ bool loadHSI (FILE * fp, int x, int y, bool reserve) {
 
 			glClear(GL_COLOR_BUFFER_BIT);	// Clear The Screen
 
-			const GLint vertices[] = { 
-				-xoffset, -yoffset, 0, 
-				realPicWidth-xoffset, -yoffset, 0, 
-				realPicWidth-xoffset, -yoffset+realPicHeight, 0, 
-				-xoffset, -yoffset+realPicHeight, 0
+			const GLfloat vertices[] = { 
+				(GLfloat)-xoffset, (GLfloat)-yoffset, 0., 
+				(GLfloat)realPicWidth-xoffset, (GLfloat)-yoffset, 0., 
+				(GLfloat)-xoffset, (GLfloat)-yoffset+realPicHeight, 0.,
+				(GLfloat)realPicWidth-xoffset, (GLfloat)-yoffset+realPicHeight, 0.
 			};
 
 			const GLfloat texCoords[] = { 
 				0.0f, 0.0f,
 				texCoordW, 0.0f,
-				texCoordW, texCoordH, 
-				0.0f, texCoordH
+				0.0f, texCoordH,
+				texCoordW, texCoordH
 			}; 
 
 			if (backdropExists) {
 				// Render the sprite to the backdrop
 				// (using mulitexturing, so the old backdrop is seen where alpha < 1.0)
-				glClientActiveTexture(GL_TEXTURE2);
 				glActiveTexture(GL_TEXTURE2);
 				glBindTexture (GL_TEXTURE_2D, backdropTextureName);
-				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-				glTexCoordPointer(2, GL_FLOAT, 0, btexCoords);
-				glClientActiveTexture(GL_TEXTURE0);
 				glActiveTexture(GL_TEXTURE0);
 
 				glUseProgram(shader.paste);
 				GLint uniform = glGetUniformLocation(shader.paste, "useLightTexture");
 				if (uniform >= 0) glUniform1i(uniform, 0); // No lighting
 
-				glColor4f(1.0, 1.0, 1.0, 1.0);
+				setPMVMatrix(shader.paste);
+
+				setPrimaryColor(1.0, 1.0, 1.0, 1.0);
 				glBindTexture(GL_TEXTURE_2D, tmpTex);
-				glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+				//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 
-				glEnableClientState(GL_VERTEX_ARRAY);
-				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-				glVertexPointer(3, GL_INT, 0, vertices);
-				glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
-
-				glDrawArrays(GL_QUADS, 0, 4);
-
-				glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-				glDisableClientState(GL_VERTEX_ARRAY);
-
-				glClientActiveTexture(GL_TEXTURE2);
-				glActiveTexture(GL_TEXTURE2);
-				glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-				glClientActiveTexture(GL_TEXTURE0);
-				glActiveTexture(GL_TEXTURE0);
+				drawQuad(shader.paste, vertices, 3, texCoords, NULL, btexCoords);
 
 				glUseProgram(0);
 
 			} else {
 				// It's all new - nothing special to be done.
 
+				glUseProgram(shader.texture);
+				setPMVMatrix(shader.texture);
+
 				glBindTexture(GL_TEXTURE_2D, tmpTex);
 
-				glColor4f(1.0, 0.0, 0.0, 0.0);
+				setPrimaryColor(1.0, 0.0, 0.0, 0.0);
 
-				glEnableClientState(GL_VERTEX_ARRAY);
-				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+				drawQuad(shader.texture, vertices, 1, texCoords);
 
-				glVertexPointer(3, GL_INT, 0, vertices);
-				glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
-
-				glDrawArrays(GL_QUADS, 0, 4);
-
-				glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-				glDisableClientState(GL_VERTEX_ARRAY);
+				glUseProgram(0);
 			}
 
 			// Copy Our ViewPort To The Texture
-			glBindTexture(GL_TEXTURE_2D, backdropTextureName);
-			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, x+xoffset, y+yoffset, viewportOffsetX, viewportOffsetY, w, h);
+			copyTexSubImage2D(GL_TEXTURE_2D, 0, x+xoffset, y+yoffset, viewportOffsetX, viewportOffsetY, w, h, backdropTextureName);
 
 			yoffset += viewportHeight;
 		}
 
 		xoffset += viewportWidth;
 	}
-	glDeleteTextures(1, &tmpTex);
+	deleteTextures(1, &tmpTex);
 
 	setPixelCoords (false);
 
@@ -1348,15 +1300,15 @@ bool mixHSI (FILE * fp, int x, int y) {
 	const GLfloat texCoords[] = { 
 		tx1, ty1,
 		tx2, ty1,
-		tx2, ty2, 
-		tx1, ty2
+		tx1, ty2,
+		tx2, ty2
 	}; 
 
 	const GLfloat btexCoords[] = { 
 		btx1, bty1,
 		btx2, bty1,
-		btx2, bty2, 
-		btx1, bty2
+		btx1, bty2,
+		btx2, bty2
 	}; 
 
 	int t1, t2, n;
@@ -1417,10 +1369,10 @@ bool mixHSI (FILE * fp, int x, int y) {
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 	}
 
-	glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, backdropTexture);
+	texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, backdropTexture, tmpTex);
 
-	glEnable (GL_TEXTURE_2D);
-	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+
+	//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 
 	setPixelCoords (true);
 
@@ -1437,58 +1389,40 @@ bool mixHSI (FILE * fp, int x, int y) {
 
 			// Render the sprite to the backdrop
 			// (using mulitexturing, so the backdrop is seen where alpha < 1.0)
-			glClientActiveTexture(GL_TEXTURE2);
 			glActiveTexture(GL_TEXTURE2);
 			glBindTexture (GL_TEXTURE_2D, backdropTextureName);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-			glTexCoordPointer(2, GL_FLOAT, 0, btexCoords);
-			glClientActiveTexture(GL_TEXTURE0);
 			glActiveTexture(GL_TEXTURE0);
 
 			glUseProgram(shader.paste);
 			GLint uniform = glGetUniformLocation(shader.paste, "useLightTexture");
 			if (uniform >= 0) glUniform1i(uniform, 0); // No lighting
 
-			glColor4f(1.0, 1.0, 1.0, 0.5);
-			glBindTexture(GL_TEXTURE_2D, tmpTex);
-			glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+			setPMVMatrix(shader.paste);
 
-			const GLint vertices[] = { 
-				-xoffset, -yoffset, 0, 
-				realPicWidth-xoffset, -yoffset, 0, 
-				realPicWidth-xoffset, -yoffset+realPicHeight, 0, 
-				-xoffset, -yoffset+realPicHeight, 0
+			setPrimaryColor(1.0, 1.0, 1.0, 0.5);
+			glBindTexture(GL_TEXTURE_2D, tmpTex);
+			//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+
+			const GLfloat vertices[] = { 
+				(GLfloat)-xoffset, (GLfloat)-yoffset, 0., 
+				(GLfloat)realPicWidth-xoffset, (GLfloat)-yoffset, 0., 
+				(GLfloat)-xoffset, (GLfloat)-yoffset+realPicHeight, 0.,
+				(GLfloat)realPicWidth-xoffset, (GLfloat)-yoffset+realPicHeight, 0.
 			};
 
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-			glVertexPointer(3, GL_INT, 0, vertices);
-			glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
-
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
-
-			glClientActiveTexture(GL_TEXTURE2);
-			glActiveTexture(GL_TEXTURE2);
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glClientActiveTexture(GL_TEXTURE0);
-			glActiveTexture(GL_TEXTURE0);
+			drawQuad(shader.paste, vertices, 3, texCoords, NULL, btexCoords);
 
 			// Copy Our ViewPort To The Texture
-			glBindTexture(GL_TEXTURE_2D, backdropTextureName);
 			glUseProgram(0);
 
-			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, (int) ((x<0) ? xoffset: x+xoffset), (int) ((y<0) ? yoffset: y+yoffset), (int) ((x<0) ?viewportOffsetX-x:viewportOffsetX), (int) ((y<0) ?viewportOffsetY-y:viewportOffsetY), w, h);
+			copyTexSubImage2D(GL_TEXTURE_2D, 0, (int) ((x<0) ? xoffset: x+xoffset), (int) ((y<0) ? yoffset: y+yoffset), (int) ((x<0) ?viewportOffsetX-x:viewportOffsetX), (int) ((y<0) ?viewportOffsetY-y:viewportOffsetY), w, h, backdropTextureName);
 
 			yoffset += viewportHeight;
 		}
 
 		xoffset += viewportWidth;
 	}
-	glDeleteTextures(1, &tmpTex);
+	deleteTextures(1, &tmpTex);
 	setPixelCoords (false);
 
 	return true;
@@ -1498,23 +1432,24 @@ void saveCorePNG  (FILE * writer, GLuint texture, int w, int h) {
 	GLint tw, th;
 
 	glBindTexture (GL_TEXTURE_2D, texture);
-	glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &tw);
-	glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &th);
+	getTextureDimensions(texture, &tw, &th);
 
 	GLubyte* image = new GLubyte [tw*th*4];
+	if (! checkNew (image)) return;
+
 	glPixelStorei (GL_PACK_ALIGNMENT, 1);
 //	glGetTexImage(GL_TEXTURE_2D, 0, GL_RGB, GL_UNSIGNED_BYTE, image);
 
 	setPixelCoords (true);
 		
-	glEnable (GL_TEXTURE_2D);
-	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+
+	//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 
 	const GLfloat texCoords[] = { 
 		0.0f, 0.0f,
 		1.0f, 0.0f,
-		1.0f, 1.0f, 
-		0.0f, 1.0f
+		0.0f, 1.0f,
+		1.0f, 1.0f
 	}; 
 
 	int xoffset = 0;
@@ -1527,23 +1462,19 @@ void saveCorePNG  (FILE * writer, GLuint texture, int w, int h) {
 			
 			glClear(GL_COLOR_BUFFER_BIT);	// Clear The Screen
 
-			const GLint vertices[] = { 
-				-xoffset, -yoffset, 0, 
-				tw-xoffset, -yoffset, 0, 
-				tw-xoffset, -yoffset+th, 0, 
-				-xoffset, -yoffset+th, 0
+			const GLfloat vertices[] = { 
+				(GLfloat)-xoffset, (GLfloat)-yoffset, 0., 
+				(GLfloat)tw-xoffset, (GLfloat)-yoffset, 0., 
+				(GLfloat)-xoffset, (GLfloat)-yoffset+th, 0.,
+				(GLfloat)tw-xoffset, (GLfloat)-yoffset+th, 0.
 			};
 
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+			glUseProgram(shader.texture);
+			setPMVMatrix(shader.texture);
 
-			glVertexPointer(3, GL_INT, 0, vertices);
-			glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
+			drawQuad(shader.texture, vertices, 1, texCoords);
 
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
+			glUseProgram(0);
 
 			for (int i = 0; i<h; i++)	{
 				glReadPixels(viewportOffsetX, viewportOffsetY+i, w, 1, GL_RGBA, GL_UNSIGNED_BYTE, image+xoffset*4+(yoffset+i)*4*tw);
@@ -1593,26 +1524,26 @@ void saveCorePNG  (FILE * writer, GLuint texture, int w, int h) {
 void saveCoreHSI (FILE * writer, GLuint texture, int w, int h) {
 
 	GLint tw, th;
-
 	glBindTexture (GL_TEXTURE_2D, texture);
-	glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &tw);
-	glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &th);
+	getTextureDimensions(texture, &tw, &th);
 
 	GLushort* image = new GLushort [tw*th];
+	if (! checkNew (image)) return;
+
 	glPixelStorei (GL_PACK_ALIGNMENT, 1);
 //	glGetTexImage(GL_TEXTURE_2D, 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, image);
 
 	setPixelCoords (true);
 	
 	
-	glEnable (GL_TEXTURE_2D);
-	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+
+	//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 	
 	const GLfloat texCoords[] = { 
 		0.0f, 0.0f,
 		1.0f, 0.0f,
-		1.0f, 1.0f, 
-		0.0f, 1.0f
+		0.0f, 1.0f,
+		1.0f, 1.0f
 	}; 
 
 	int xoffset = 0;
@@ -1625,23 +1556,17 @@ void saveCoreHSI (FILE * writer, GLuint texture, int w, int h) {
 			
 			glClear(GL_COLOR_BUFFER_BIT);	// Clear The Screen
 
-			const GLint vertices[] = { 
-				-xoffset, -yoffset, 0, 
-				w-xoffset, -yoffset, 0, 
-				w-xoffset, -yoffset+h, 0, 
-				-xoffset, -yoffset+h, 0
+			const GLfloat vertices[] = { 
+				(GLfloat)-xoffset, (GLfloat)-yoffset, 0., 
+				(GLfloat)w-xoffset, (GLfloat)-yoffset, 0., 
+				(GLfloat)-xoffset, (GLfloat)-yoffset+h, 0.,
+				(GLfloat)w-xoffset, (GLfloat)-yoffset+h, 0.
 			};
 
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-			glVertexPointer(3, GL_INT, 0, vertices);
-			glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
-
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
+			glUseProgram(shader.texture);
+			setPMVMatrix(shader.texture);
+			drawQuad(shader.texture, vertices, 1, texCoords);
+			glUseProgram(0);
 			
 			for (int i = 0; i<h; i++)	{
 				glReadPixels(viewportOffsetX, viewportOffsetY+i, w, 1, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, image+xoffset+(yoffset+i)*tw);
