@@ -35,6 +35,7 @@
 #include "language.h"
 #include "thumbnail.h"
 #include "graphics.h"
+#include "utf8.h"
 
 extern char * gamePath;
 
@@ -63,7 +64,6 @@ extern char * * allUserFunc;
 extern char * * allBIFNames;
 extern inputType input;
 extern char * loadNow;
-extern byte fontTable[256];
 
 extern GLuint backdropTextureName;
 
@@ -514,7 +514,7 @@ builtIn(substring)
 	UNUSEDALL
 	char * wholeString;
 	char * newString;
-	int start, length, a;
+	int start, length;
 
 	//debugOut ("BUILTIN: substring\n");
 
@@ -524,23 +524,30 @@ builtIn(substring)
 	trimStack (fun -> stack);
 	wholeString = getTextFromAnyVar (fun -> stack -> thisVar);
 	trimStack (fun -> stack);
-	//debugOut ("String is %s (%i)\n", wholeString, length);
+	
+	if (u8_strlen(wholeString) < start+length) {
+		length = u8_strlen(wholeString) - start;
+		if (u8_strlen(wholeString) < start) {
+			start = 0;
+		}
+	}
 	if (length<0) {
 		length=0;
 	}
+	
+	int startoffset = u8_offset (wholeString, start);
+	int endoffset = u8_offset (wholeString, start+length);
 
-	newString = new char[length + 1];
+	newString = new char[endoffset - startoffset + 1];
 	if (! checkNew (newString)) {
-		//debugOut ("BUILTIN: substring exiting early.\n");
 		return BR_ERROR;
 	}
-
-	for (a = 0; a < length; a ++) newString[a] = wholeString[start ++];
-
-	newString[length] = 0;
+	
+	memcpy (newString, wholeString + startoffset, endoffset - startoffset);
+	newString[endoffset - startoffset] = 0;
+	
 	makeTextVar (fun -> reg, newString);
 	delete newString;
-	//debugOut ("BUILTIN: substring exiting normally.\n");
 	return BR_CONTINUE;
 }
 
@@ -549,7 +556,7 @@ builtIn(stringLength)
 	UNUSEDALL
 	char * newText = getTextFromAnyVar (fun -> stack -> thisVar);
 	trimStack (fun -> stack);
-	setVariable (fun -> reg, SVT_INT, strlen (newText));
+	setVariable (fun -> reg, SVT_INT, stringLength(newText));
 	delete newText;
 	return BR_CONTINUE;
 }
@@ -878,7 +885,8 @@ builtIn(inFont)
 	trimStack (fun -> stack);
 
 	// Return value
-	setVariable (fun -> reg, SVT_INT, newText[0] && newText[1] == 0 && fontTable[(unsigned char) newText[0]] != 0);
+	
+	setVariable (fun -> reg, SVT_INT, isInFont(newText));
 	return BR_CONTINUE;
 }
 
@@ -1069,7 +1077,7 @@ builtIn (playMovie)
 {
 	UNUSEDALL
 	int fileNumber, r;
-	
+		
 	if (movieIsPlaying) return BR_PAUSE;
 	
 	if (! getValueType (fileNumber, SVT_FILE, fun -> stack -> thisVar)) return BR_ERROR;
@@ -1079,7 +1087,7 @@ builtIn (playMovie)
 	
 	setVariable (fun -> reg, SVT_INT, r);
 	
-	if (! fun->next) {
+	if (r && (! fun->next)) {
 		restartFunction (fun);
 		return BR_ALREADY_GONE;
 	}
@@ -1200,6 +1208,8 @@ builtIn(loopSound)
 				return BR_ERROR;
 			}
 			s = new soundList;
+			if (! checkNew (s)) return BR_ERROR;
+
 			s-> next = old;
 			s-> prev = NULL;
 			s-> sound = fileNumber;
@@ -2664,7 +2674,7 @@ char builtInFunctionNames[][25] =
 
 
 builtReturn callBuiltIn (int whichFunc, int numParams, loadedFunction * fun) {
-    //fprintf (stderr, "Calling function %d: %s\n", whichFunc, builtInFunctionNames[whichFunc]);    fflush (stderr);
+   // fprintf (stderr, "Calling function %d: %s\n", whichFunc, builtInFunctionNames[whichFunc]);    fflush (stderr);
 	if (numBIFNames) {
 
 		//		deb ("IN:", (fun -> originalNumber < numUserFunc) ? allUserFunc[fun -> originalNumber] : "Unknown user function");
