@@ -1,3 +1,4 @@
+#include <stdarg.h>
 #include <SDL/SDL.h>
 
 #include "debug.h"
@@ -5,13 +6,26 @@
 #include "specialsettings.h"
 #include "graphics.h"
 #include "language.h"
+#include "newfatal.h"
 #include "sprbanks.h"
 #include "zbuffer.h"
 #include "backdrop.h"
 #include "shaders.h"
+#include "movie.h"
+#include "stringy.h"
 
 #include "language.h" // for settings
 
+#if !defined(HAVE_GLES2)
+#ifdef _WIN32
+#include <GL\glu.h> // handy for gluErrorString
+#elif defined __APPLE__
+#include <OpenGL/glu.h>
+#else
+#include <GL/glu.h>
+#endif
+#endif
+
 unsigned int winWidth, winHeight;
 int viewportHeight, viewportWidth;
 int viewportOffsetX = 0, viewportOffsetY = 0;
@@ -23,7 +37,6 @@ bool NPOT_textures = true;
 extern int specialSettings;
 
 void setMovieViewport();
-extern int movieIsPlaying;
 
 extern GLuint backdropTextureName;
 extern GLuint snapshotTextureName;
@@ -35,12 +48,203 @@ extern int lightMapNumber;
 extern GLuint yTextureName;
 extern GLuint uTextureName;
 extern GLuint vTextureName;
-extern GLubyte * ytex, * utex, * vtex;
+//extern GLubyte * ytex, * utex, * vtex;
 
 shaders shader;
+GLfloat aPMVMatrix[16];
 
 void sludgeDisplay ();
 
+GLfloat primaryColor[4];
+GLfloat secondaryColor[4];
+
+struct textureList *firstTexture = NULL;
+
+textureList * addTexture () {
+	textureList * newTexture = new textureList;
+	newTexture -> next = firstTexture;
+	firstTexture = newTexture;
+	return newTexture;
+}
+
+void deleteTextures(GLsizei n,  const GLuint * textures)
+{
+	if (firstTexture == NULL) {
+		//debugOut("Deleting texture while list is already empty.\n");
+	} else {
+		for (int i = 0; i < n; i++) {
+			bool found = false;
+			textureList *list = firstTexture;
+			if (list->name == textures[i]) {
+				found = true;
+				firstTexture = list->next;
+				delete list;
+				continue;
+			}
+
+			while (list->next) {
+				if (list->next->name == textures[i]) {
+					found = true;
+					textureList *deleteMe = list->next;
+					list->next = list->next->next;
+					delete deleteMe;
+					break;
+				}
+				list = list->next;
+			}
+			//if (!found)
+			//	debugOut("Deleting texture that was not in list.\n");
+		}
+	}
+
+	glDeleteTextures(n, textures);
+}
+
+void getTextureDimensions(GLuint name, GLint *width,  GLint *height)
+{
+	textureList *list = firstTexture;
+	while (list) {
+		if (list->name == name)  {
+			*width = list->width;
+			*height = list->height;
+#if !defined(HAVE_GLES2)
+			//For the following test it is assumed that glBindTexture is always 
+			//called for the right texture before getTextureDimensions.
+			GLint tw, th;
+			glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &tw); 
+			glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &th);
+			if (tw != *width || th != *height) {
+				debugOut ("Warning: Texture dimensions don't match: They are %ix%i, but SLUDGEs bookkeeping says %ix%i.\n", tw, th, *width, *height);
+			}
+#endif
+			return;
+		}
+		list = list->next;
+	}
+	fatal("Texture not found in list.\n");
+}
+
+void storeTextureDimensions(GLuint name, GLsizei width,  GLsizei height, const char *file, int line)
+{
+	if (! NPOT_textures && !(((height & (height - 1)) == 0) || ((width & (width - 1)) == 0))) {
+		debugOut("I was told to create a texture with dimensions %ix%i in %s @ line %d although NPOT textures are disabled.\n", width, height, file, line);
+		//height = getNextPOT(height);
+		//width = getNextPOT(width);
+	}
+
+	textureList *list = firstTexture;
+	while (list) {
+		if (list->name == name)  {
+			//debugOut("Texture dimensions are overwritten.\n");
+			break;
+		}
+		list = list->next;
+	}
+	if (list == NULL) {
+		list = addTexture();
+	}
+	list->name = name;
+	list->width = width;
+	list->height = height;
+}
+
+
+void dcopyTexImage2D(GLenum target,  GLint level,  GLenum internalformat,  GLint x,  GLint y,  GLsizei width,  GLsizei height,  GLint border, GLuint name, const char *file, int line)
+{
+	glBindTexture(GL_TEXTURE_2D, name);
+	glCopyTexImage2D(target, level, internalformat, x, y, width, height, border);
+}
+
+void dcopyTexSubImage2D(GLenum target,  GLint level,  GLint xoffset,  GLint yoffset,  GLint x,  GLint y,  GLsizei width,  GLsizei height, GLuint name, const char *file, int line)
+{
+	glBindTexture(GL_TEXTURE_2D, name);
+	glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height);
+}
+
+void dtexImage2D(GLenum target,  GLint level,  GLint internalformat,  GLsizei width,  GLsizei height, 
+		GLint border,  GLenum format,  GLenum type,  const GLvoid * data, GLuint name, const char *file, int line)
+{
+	storeTextureDimensions(name, width,  height, file, line);
+	glBindTexture(GL_TEXTURE_2D, name);
+	glTexImage2D(target, level, internalformat, width, height, border, format, type,  data);
+}
+
+void dtexSubImage2D(GLenum target,  GLint level,  GLint xoffset,  GLint yoffset,  GLsizei width,  GLsizei height,
+		GLenum format,  GLenum type,  const GLvoid * data, GLuint name, const char *file, int line) 
+{
+	storeTextureDimensions(name, width,  height, file, line);
+	glBindTexture(GL_TEXTURE_2D, name);
+	glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, data);
+}
+
+void setPrimaryColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
+{
+	primaryColor[0] = r;
+	primaryColor[1] = g;
+	primaryColor[2] = b;
+	primaryColor[3] = a;
+}
+
+void setSecondaryColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
+{
+	secondaryColor[0] = r;
+	secondaryColor[1] = g;
+	secondaryColor[2] = b;
+	secondaryColor[3] = a;
+}
+
+void drawQuad(GLint program, const GLfloat* vertices, int numTexCoords, ...)
+{
+	int i, vertexLoc, texCoordLocs[numTexCoords];
+	const GLfloat* texCoords[numTexCoords];
+
+	va_list vl;
+	va_start(vl,numTexCoords);
+	for (i=0;i<numTexCoords;i++)
+	{
+		texCoords[i]=va_arg(vl,const GLfloat*);
+	}
+	va_end(vl);
+
+	glUniform4f(glGetUniformLocation(program, "myColor"), primaryColor[0], primaryColor[1], primaryColor[2], primaryColor[3]);
+	if (program == shader.smartScaler || program == shader.paste)
+	{
+		glUniform4f(glGetUniformLocation(program, "mySecondaryColor"), secondaryColor[0], secondaryColor[1], secondaryColor[2], secondaryColor[3]);
+	}
+
+	vertexLoc = glGetAttribLocation(program, "myVertex");
+	texCoordLocs[0] = glGetAttribLocation(program, "myUV0");
+	if (numTexCoords > 1) texCoordLocs[1] = glGetAttribLocation(program, "myUV1");
+	if (numTexCoords > 2) texCoordLocs[2] = glGetAttribLocation(program, "myUV2");
+	if (numTexCoords > 3) texCoordLocs[3] = glGetAttribLocation(program, "myUV3");
+ 		
+	glEnableVertexAttribArray(vertexLoc);
+	glVertexAttribPointer(vertexLoc, 3, GL_FLOAT, GL_FALSE, 0, vertices);
+
+	for (i=0;i<numTexCoords;i++)
+	{
+		if (texCoords[i]) {
+			glEnableVertexAttribArray(texCoordLocs[i]);
+			glVertexAttribPointer(texCoordLocs[i], 2, GL_FLOAT, GL_FALSE, 0, texCoords[i]);
+		}
+	}
+
+	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+
+	for (i=0;i<numTexCoords;i++)
+	{
+		if (texCoords[i]) {
+			glDisableVertexAttribArray(texCoordLocs[i]);
+		}
+	}
+
+	glDisableVertexAttribArray(vertexLoc);
+}
+
+
+void setPMVMatrix(GLint program) {
+	glUniformMatrix4fv( glGetUniformLocation(program, "myPMVMatrix"), 1, GL_FALSE, aPMVMatrix);
+}
 
 // This is for swapping settings between rendering to texture or to the screen
 void setPixelCoords (bool pixels) {
@@ -53,13 +257,21 @@ void setPixelCoords (bool pixels) {
 	if (pixels) {
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-		glMatrixMode(GL_PROJECTION);
-		glLoadIdentity();
-		glOrtho(0, viewportWidth, 0, viewportHeight, 1.0, -1.0);
-
-		glMatrixMode(GL_MODELVIEW);
 
 		glClear(GL_COLOR_BUFFER_BIT);
+
+		const GLfloat bPMVMatrix[] =
+		{
+		2.0f/viewportWidth,                  .0,   .0,  .0,
+			        .0, 2.0f/viewportHeight,   .0,  .0,
+			        .0,                  .0, 1.0f,  .0,
+			      -1.0,               -1.0f,   .0, 1.0f
+
+		};
+		for (int i = 0; i < 16; i++)
+		{
+			aPMVMatrix[i] = bPMVMatrix[i];
+		}
 	} else {
 		if (gameSettings.antiAlias < 0) {
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
@@ -69,16 +281,21 @@ void setPixelCoords (bool pixels) {
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 		}
 
-		glMatrixMode(GL_PROJECTION);
-		glLoadIdentity();
-		GLdouble w = (GLdouble) winWidth / cameraZoom;
-		GLdouble h = (GLdouble) winHeight / cameraZoom;
+		GLfloat w = (GLfloat) winWidth / cameraZoom;
+		GLfloat h = (GLfloat) winHeight / cameraZoom;
 
-		glOrtho(0, w, h, 0, 1.0, -1.0);
+		const GLfloat bPMVMatrix[] =
+		{
+		2.0f/w,      .0,   .0,  .0,
+		    .0, -2.0f/h,   .0,  .0,
+		    .0,      .0, 1.0f,  .0,
+		  -1.0,    1.0f,   .0, 1.0f
 
-//		glOrtho(0, winWidth, winHeight, 0, 1.0, -1.0);
-
-		glMatrixMode(GL_MODELVIEW);
+		};
+		for (int i = 0; i < 16; i++)
+		{
+			aPMVMatrix[i] = bPMVMatrix[i];
+		}
 	}
 }
 
@@ -86,7 +303,7 @@ int desktopW = 0, desktopH = 0;
 bool runningFullscreen = false;
 
 
-#ifdef _WIN32
+#if defined _WIN32 || defined(HAVE_GLES2)
 // Replacement for glGetTexImage, because some ATI drivers are buggy.
 void saveTexture (GLuint tex, GLubyte * data) {
 	setPixelCoords (true);
@@ -94,12 +311,9 @@ void saveTexture (GLuint tex, GLubyte * data) {
 	glBindTexture (GL_TEXTURE_2D, tex);
 
 	GLint tw, th;
+	getTextureDimensions(tex, &tw, &th);
 
-	glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &tw);
-	glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &th);
-
-	glEnable (GL_TEXTURE_2D);
-	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+	//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 
 	int xoffset = 0;
 	while (xoffset < tw) {
@@ -111,30 +325,25 @@ void saveTexture (GLuint tex, GLubyte * data) {
 
 			glClear(GL_COLOR_BUFFER_BIT);	// Clear The Screen
 
-			const GLint vertices[] = { 
-				-xoffset, -yoffset, 0, 
-				tw-xoffset, -yoffset, 0, 
-				tw-xoffset, -yoffset+th, 0, 
-				-xoffset, -yoffset+th, 0
+			const GLfloat vertices[] = { 
+				(GLfloat)-xoffset, (GLfloat)-yoffset, 0.f, 
+				(GLfloat)tw-xoffset, (GLfloat)-yoffset, 0.f, 
+				(GLfloat)-xoffset, (GLfloat)-yoffset+th, 0.f,
+				(GLfloat)tw-xoffset, (GLfloat)-yoffset+th, 0.f
 			};
 
 			const GLfloat texCoords[] = { 
 				0.0f, 0.0f,
 				1.0f, 0.0f,
-				1.0f, 1.0f, 
-				0.0f, 1.0f
+				0.0f, 1.0f,
+				1.0f, 1.0f
 			}; 
-	
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 
-			glVertexPointer(3, GL_INT, 0, vertices);
-			glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
+			glUseProgram(shader.texture);
+			setPMVMatrix(shader.texture);
 
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
+			drawQuad(shader.texture, vertices, 1, texCoords);
+			glUseProgram(0);
 
 			for (int i = 0; i<h; i++)	{
 				glReadPixels(viewportOffsetX, viewportOffsetY+i, w, 1, GL_RGBA, GL_UNSIGNED_BYTE, data+xoffset*4+(yoffset+i)*4*tw);
@@ -187,6 +396,7 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
 				picHeight = getNextPOT(picHeight);
 			}
 			backdropTexture = new GLubyte [picHeight*picWidth*4];
+			if (! checkNew (backdropTexture)) return;			
 
 			saveTexture (backdropTextureName, backdropTexture);
 		}
@@ -198,6 +408,7 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
 				picHeight = getNextPOT(picHeight);
 			}
 			snapTexture = new GLubyte [picHeight*picWidth*4];
+			if (! checkNew (snapTexture)) return;
 
 			saveTexture (snapshotTextureName, snapTexture);
 		}
@@ -210,7 +421,11 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
 
 	if (fullscreen) {
 		specialSettings &= ~SPECIAL_INVISIBLE;
-		videoflags = SDL_OPENGL | SDL_FULLSCREEN;
+#if !defined(HAVE_GLES2)
+ 		videoflags = SDL_OPENGL | SDL_FULLSCREEN;
+#else
+		videoflags = SDL_SWSURFACE | SDL_FULLSCREEN;
+#endif
 
         if (gameSettings.fixedPixels) {
             viewportWidth = realWinWidth = winWidth;
@@ -237,7 +452,11 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
         }
 
 	} else {
-		videoflags = SDL_OPENGL/* | SDL_RESIZABLE*/;
+#if !defined(HAVE_GLES2)
+ 		videoflags = SDL_OPENGL/* | SDL_RESIZABLE*/;
+#else
+		videoflags = SDL_SWSURFACE;
+#endif
 
 		if (resize) {
             float realAspect = (float) desktopW / desktopH;
@@ -289,24 +508,43 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
 
 	if( SDL_SetVideoMode( realWinWidth, realWinHeight, 32, videoflags ) == 0 ) {
 		msgBox("Startup Error: Couldn't set video mode.", SDL_GetError());
+#if defined(HAVE_GLES2)
+		EGL_Close();
+#endif
 		SDL_Quit();
 		exit(2);
 	}
 	debugOut( "Video mode %d %d set successfully.\n", realWinWidth, realWinHeight);
 
+#if defined(HAVE_GLES2)
+	EGL_Init();
+#endif
+
 	GLint uniform;
-
-
-    const GLchar *Vertex;
-    const GLchar *Fragment;
-
-	if (gameSettings.antiAlias > 0) {
+	const char *Vertex;
+	const char *Fragment;
 
         Vertex = shaderFileRead("scale.vert");
+
+#if !defined(HAVE_GLES2)
         Fragment = shaderFileRead("scale.frag");
+#else
+/*	const GLubyte *str;
+	int glDerivativesAvailable;
+	str = glGetString (GL_EXTENSIONS);
+	glDerivativesAvailable = (strstr((const char *)str, "GL_OES_standard_derivatives") != NULL);
+	if (!glDerivativesAvailable) {
+		debugOut("Extension \"GL_OES_standard_derivatives\" not available. Advanced anti-aliasing is not possible. Using linear anti-aliasing instead.");
+		gameSettings.antiAlias = -1;
+*/
+        	Fragment = shaderFileRead("scale_noaa.frag");
+//	}
+
+	Fragment = joinStrings("precision mediump float;\n", Fragment);
+#endif
 
         if (! Vertex || ! Fragment) {
-            msgBox ("Error loading \"scale\" shader program!", "Try re-installing the game. Advanced anti-aliasing is not possible. Using linear anti-aliasing instead.");
+            fatal ("Error loading \"scale\" shader program!", "Try re-installing the game. (scale.frag, scale_noaa.frag or scale.vert was not found.)");
             gameSettings.antiAlias = -1;
             shader.smartScaler = 0;
         } else {
@@ -314,7 +552,7 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
             shader.smartScaler = buildShaders (Vertex, Fragment);
 			
 			if (! shader.smartScaler) {
-				msgBox ("Error building \"scale\" shader program!", "Advanced anti-aliasing is not possible. Using linear anti-aliasing instead.");
+				fatal ("Error building \"scale\" shader program!");
 				gameSettings.antiAlias = -1;
 				shader.smartScaler = 0;
 			} else {
@@ -326,25 +564,30 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
 				if (uniform >= 0) glUniform1i(uniform, 1);
 				uniform = glGetUniformLocation(shader.smartScaler, "useLightTexture");
 				if (uniform >= 0) glUniform1i(uniform, 0);
+				uniform = glGetUniformLocation(shader.smartScaler, "antialias");
+				if (uniform >= 0) glUniform1i(uniform, 0);
 				uniform = glGetUniformLocation(shader.smartScaler, "scale");
 				float scale = (float)realWinWidth/(float)winWidth*0.25;
 				if (scale > 1.0) scale = 1.0;
 				if (uniform >= 0) glUniform1f(uniform, scale);
 			}
         }
-	}
 
 	Vertex = shaderFileRead("fixScaleSprite.vert");
 	Fragment = shaderFileRead("fixScaleSprite.frag");
 
+#if defined(HAVE_GLES2)
+	Fragment = joinStrings("precision mediump float;\n", Fragment);
+#endif
+
 	if (! Vertex || ! Fragment) {
-		msgBox( "Error loading \"fixScaleSprite\" shader program!", "Try re-installing the game. The game will run anyway, but some graphics may be corrupted.");
+		fatal ("Error loading \"fixScaleSprite\" shader program!", "Try re-installing the game. (fixScaleSprite.frag or fixScaleSprite.vert was not found.)");
 		shader.paste = 0;
 	} else {
 
 		shader.paste = buildShaders (Vertex, Fragment);
 		if (! shader.paste) {
-			msgBox( "Error building \"fixScaleSprite\" shader program!", "Try updating the drivers for your graphics card. If that doesn't help - sorry, your graphics card simply doesn't have all features needed for this game. It will run anyway, but some graphics may be corrupted.");
+			fatal( "Error building \"fixScaleSprite\" shader program!");
 		} else {
 			debugOut( "Built shader program: %d (fixScaleSprite)\n", shader.paste);
 			glUseProgram(shader.paste);
@@ -362,14 +605,18 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
 	Vertex = shaderFileRead("yuv.vert");
 	Fragment = shaderFileRead("yuv.frag");
 
+#if defined(HAVE_GLES2)
+	Fragment = joinStrings("precision mediump float;\n", Fragment);
+#endif
+
 	if (! Vertex || ! Fragment) {
-		msgBox( "Error loading \"yuv\" shader program!", "Try re-installing the game. It will run anyway, but any movies will be greyscale.");
+		fatal ("Error loading \"yuv\" shader program!", "Try re-installing the game. (yuv.frag or yuv.vert was not found.)");
 		shader.yuv = 0;
 	} else {
 
 		shader.yuv = buildShaders (Vertex, Fragment);
 		if (! shader.yuv) {
-			msgBox( "Error building \"yuv\" shader program!", "Try updating the drivers for your graphics card. If that doesn't help - sorry, your graphics card simply doesn't have all features needed for this game. It will run anyway, but any movies will be greyscale.");			
+			fatal( "Error building \"yuv\" shader program!");			
 		} else {
 			debugOut( "Built shader program: %d (yuv)\n", shader.yuv);
 			glUseProgram(shader.yuv);
@@ -382,6 +629,56 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
 		}
 	}
 
+	Vertex = shaderFileRead("texture.vert");
+	Fragment = shaderFileRead("texture.frag");
+
+#if defined(HAVE_GLES2)
+	Fragment = joinStrings("precision mediump float;\n", Fragment);
+#endif
+
+	if (! Vertex || ! Fragment) {
+		fatal ("Error loading \"texture\" shader program!", "Try re-installing the game. (texture.frag or texture.vert was not found.)");
+		shader.texture = 0;
+	} else {
+
+		shader.texture = buildShaders (Vertex, Fragment);
+		if (! shader.texture) {
+			fatal( "Error building \"texture\" shader program!");
+		} else {
+			debugOut( "Built shader program: %d (texture)\n", shader.texture);
+			glUseProgram(shader.texture);
+			uniform = glGetUniformLocation(shader.texture, "sampler2d");
+			if (uniform >= 0) glUniform1i(uniform, 0);
+			uniform = glGetUniformLocation(shader.texture, "zBuffer");
+			if (uniform >= 0) glUniform1i(uniform, 0);
+			uniform = glGetUniformLocation(shader.texture, "zBufferLayer");
+			if (uniform >= 0) glUniform1f(uniform, 0.);
+			uniform = glGetUniformLocation(shader.texture, "modulateColor");
+			if (uniform >= 0) glUniform1i(uniform, 0);
+		}
+	}
+
+	Vertex = shaderFileRead("color.vert");
+	Fragment = shaderFileRead("color.frag");
+
+#if defined(HAVE_GLES2)
+	Fragment = joinStrings("precision mediump float;\n", Fragment);
+#endif
+
+	if (! Vertex || ! Fragment) {
+		fatal ("Error loading \"color\" shader program!", "Try re-installing the game. (color.frag or color.vert was not found.)");
+		shader.color = 0;
+	} else {
+
+		shader.color = buildShaders (Vertex, Fragment);
+		if (! shader.color) {
+			fatal( "Error building \"color\" shader program!");
+		} else {
+			debugOut( "Built shader program: %d (color)\n", shader.color);
+			glUseProgram(shader.color);
+		}
+	}
+
 	glUseProgram(0);
 
 	glViewport (viewportOffsetX, viewportOffsetY, viewportWidth, viewportHeight);
@@ -395,9 +692,6 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
 
 	setPixelCoords (false);
 
-	glLoadIdentity();
-	glTranslatef(0.0f, 0.0f, 0.0f);
-
 	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
 
 	if (restoreGraphics) {
@@ -421,7 +715,7 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
 			}
 
 			// Restore the backdrop
-			glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, sceneWidth, sceneHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, backdropTexture);
+			texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, sceneWidth, sceneHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, backdropTexture, backdropTextureName);
 
 		}
 		if (snapshotTextureName) {
@@ -436,19 +730,15 @@ void setGraphicsWindow(bool fullscreen, bool restoreGraphics, bool resize) {
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 
 			// Restore the backdrop
-			glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, winWidth, winHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, snapTexture);
+			texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, winWidth, winHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, snapTexture, snapshotTextureName);
 			delete snapTexture;
 		}
-		if (ytex) {
+		if (yTextureName) {
 			if (!glIsTexture(yTextureName)) {
 				glGenTextures (1, &yTextureName);
 				glGenTextures (1, &uTextureName);
 				glGenTextures (1, &vTextureName);
 			}
-			delete [] ytex;
-			delete [] utex;
-			delete [] vtex;
-			ytex = utex = vtex = NULL;
 		}
 
 		reloadSpriteTextures ();
@@ -489,6 +779,7 @@ void setupOpenGLStuff() {
 
 	setGraphicsWindow(gameSettings.userFullScreen, false);
 
+#if !defined(HAVE_GLES2)
 	/* Check for graphics capabilities... */
 	if (GLEE_VERSION_2_0) {
 		// Yes! Textures can be any size!
@@ -524,19 +815,22 @@ void setupOpenGLStuff() {
 		if (GLEE_ARB_shader_objects) {
 			debugOut("ARB_shader_objects supported.\n");
 		} else {
-			debugOut("Warning: Old graphics card! ARB_shader_objects not supported.\n");
+			fatal("Error: Old graphics card! ARB_shader_objects not supported.\n");
 		}
 		if (GLEE_ARB_vertex_shader) {
 			debugOut("ARB_vertex_shader supported.\n");
 		} else {
-			debugOut("Warning: Old graphics card! ARB_vertex_shader not supported.\n");
+			fatal("Error: Old graphics card! ARB_vertex_shader not supported.\n");
 		}
 		if (GLEE_ARB_fragment_shader) {
 			debugOut("ARB_fragment_shader supported.\n");
 		} else {
-			debugOut("Warning: Old graphics card! ARB_fragment_shader not supported.\n");
+			fatal("Error: Old graphics card! ARB_fragment_shader not supported.\n");
 		}
 	}
+#else
+	NPOT_textures = false;
+#endif
 
 	int n;
 	glGetIntegerv(GL_MAX_TEXTURE_IMAGE_UNITS, (GLint *) &n);
@@ -558,3 +852,23 @@ int getNextPOT(int n) {
 	++n;
 	return n;
 }
+
+int printOglError (const char *file, int         line)
+{
+	/* Returns 1 if an OpenGL error occurred, 0 otherwise. */
+	GLenum glErr;
+	int    retCode = 0;
+
+	glErr = glGetError ();
+	while (glErr != GL_NO_ERROR)
+    {
+#if !defined(HAVE_GLES2)
+		debugOut("glError in file %s @ line %d: %s\n", file, line, gluErrorString (glErr));
+#else
+		debugOut("glError in file %s @ line %d: error code %i\n", file, line, glErr);
+#endif
+		retCode = 1;
+		glErr = glGetError ();
+    }
+	return retCode;
+}
