@@ -10,6 +10,7 @@
 #include <SDL/SDL.h>
 #include <math.h>
 
+#include "specialsettings.h"
 
 #include "libwebm/mkvreader.hpp"
 #include "libwebm/mkvparser.hpp"
@@ -33,6 +34,9 @@
 
 #include "AL/alure.h"
 
+
+extern int specialSettings;
+
 // in main.c
 int checkInput();
 extern int weAreDoneSoQuit;
@@ -45,13 +49,10 @@ void playMovieStream (int a);
 int initMovieSound(int f, ALenum format, int audioChannels, ALuint samplerate,
 				   ALuint (*callback)(void *userdata, ALubyte *data, ALuint bytes));
 
-/*
- movieIsPlaying tracks the state of movie playing
- 0 = no movie
- 1 = movie is played
- 2 = paused
-*/
-int movieIsPlaying = 0;
+
+movieStates movieIsPlaying = nothing;
+
+int movieIsEnding = 0;
 
 float movieAspect = 1.6;
 
@@ -59,6 +60,7 @@ typedef struct audioBuffers {
 	char * buffer;
 	unsigned int size;
 	audioBuffers *next;
+	Uint32 time_ms;
 } audioBuffers;
 
 typedef struct audioQueue {
@@ -70,6 +72,8 @@ typedef struct audioQueue {
 
 audioQueue audioQ;
 
+Uint32 movieStartTick, movieCurrentTime;
+
 long long audioNsPerByte;
 long long audioNsPlayed;
 long long audioNsBuffered;
@@ -79,16 +83,32 @@ int movieAudioIndex;
 GLuint yTextureName = 0;
 GLuint uTextureName = 0;
 GLuint vTextureName = 0;
-GLubyte * ytex = NULL;
-GLubyte * utex = NULL;
-GLubyte * vtex = NULL;
+
+typedef struct videoBuffers {
+	GLubyte * ytex;
+	GLubyte * utex;
+	GLubyte * vtex;
+	videoBuffers *next;
+	GLsizei w, h;
+	Uint32 time_ms;
+} videoBuffers;
+
+typedef struct videoQueue {
+	videoBuffers *first, *last;
+	int size;
+	SDL_mutex *mutex;
+	SDL_cond *cond;
+} videoQueue;
+
+videoQueue videoQ;
+
 
 void audio_queue_init(audioQueue *q) {
 	memset(q, 0, sizeof(audioQueue));
 	q->mutex = SDL_CreateMutex();
 	q->cond = SDL_CreateCond();
 }
-int audio_queue_put(audioQueue *q, char *buffer, unsigned int size) {
+int audio_queue_put(audioQueue *q, char *buffer, unsigned int size, long long time_ms) {
 
 	audioBuffers *audioBuf = new audioBuffers;
 	if (!audioBuf)
@@ -96,6 +116,7 @@ int audio_queue_put(audioQueue *q, char *buffer, unsigned int size) {
 	audioBuf->buffer = buffer;
 	audioBuf->next = NULL;
 	audioBuf->size = size;
+	audioBuf->time_ms = time_ms;
 
 	SDL_LockMutex(q->mutex);
 
@@ -110,48 +131,105 @@ int audio_queue_put(audioQueue *q, char *buffer, unsigned int size) {
 	SDL_UnlockMutex(q->mutex);
 	return 0;
 }
-inline static int audio_queue_get(audioQueue *q, char **buffer, int block)
+inline static int audio_queue_get(audioQueue *q, char **buffer)
 {
 	audioBuffers *audioBuf;
 	int ret = 0;
 
 	SDL_LockMutex(q->mutex);
 
-//	for(;;) {
-/*
-		if(quit) {
-			ret = -1;
-			break;
+	audioBuf = q->first;
+	if (audioBuf) {
+		// Synch video timer to audio
+		Uint32 tick = SDL_GetTicks()+100;
+		if (abs((long int)((tick - movieStartTick)-(audioBuf->time_ms))) > 300) {
+			movieStartTick = tick - audioBuf->time_ms;
 		}
-*/
-		audioBuf = q->first;
-		if (audioBuf) {
-			q->first = audioBuf->next;
-			if (!q->first)
-				q->last = NULL;
-			q->size--;
-			*buffer = audioBuf->buffer;
-			ret = audioBuf->size;
-			//break;
-		} /*else if (!block) {
-			ret = 0;
-			break;
-		} else {
-			SDL_CondWait(q->cond, q->mutex);
-		}
-	}*/
+				
+		q->first = audioBuf->next;
+		if (!q->first)
+			q->last = NULL;
+		q->size--;
+		*buffer = audioBuf->buffer;
+		ret = audioBuf->size;
+		delete audioBuf;
+	} 
+	
 	SDL_UnlockMutex(q->mutex);
 
 	return ret;
 }
 
+void video_queue_init(videoQueue *q) {
+	memset(q, 0, sizeof(videoQueue));
+	q->mutex = SDL_CreateMutex();
+	q->cond = SDL_CreateCond();
+}
+int video_queue_put(videoQueue *q, GLubyte * ytex, 
+					GLubyte * utex, 
+					GLubyte * vtex,
+					GLsizei w, GLsizei h, 
+					long long time_ms) {
+	
+	videoBuffers *videoBuf = new videoBuffers;
+	if (!videoBuf)
+		return -1;
+	videoBuf->ytex = ytex;
+	videoBuf->utex = utex;
+	videoBuf->vtex = vtex;
+	videoBuf->next = NULL;
+	videoBuf->w = w;
+	videoBuf->h = h;
+	videoBuf->time_ms = time_ms;
+	
+	SDL_LockMutex(q->mutex);
+	
+	if (!q->last)
+		q->first = videoBuf;
+	else
+		q->last->next = videoBuf;
+	q->last = videoBuf;
+	q->size ++;
+	SDL_CondSignal(q->cond);
+	
+	SDL_UnlockMutex(q->mutex);
+	return 0;
+}
+inline static int video_queue_get(videoQueue *q, 
+								  GLubyte ** ytex, 
+								  GLubyte ** utex, 
+								  GLubyte ** vtex,
+								  GLsizei *w, GLsizei *h)
+{
+	videoBuffers *videoBuf;
+	int ret = 0;
+	
+	SDL_LockMutex(q->mutex);
+	
+	videoBuf = q->first;
+	if (videoBuf) {
+		q->first = videoBuf->next;
+		if (!q->first)
+			q->last = NULL;
+		q->size--;
+		*ytex = videoBuf->ytex;
+		*utex = videoBuf->utex;
+		*vtex = videoBuf->vtex;
+		*w = videoBuf->w;
+		*h = videoBuf->h;
+		ret = 1;
+		delete videoBuf;
+	} 
+	
+	SDL_UnlockMutex(q->mutex);
+	
+	return ret;
+}
+
 
 static void die_codec(vpx_codec_ctx_t *ctx, const char *s) {
     //const char *detail = vpx_codec_error_detail(ctx);
-    printf("%s: %s\n", s, vpx_codec_error(ctx));
 	fatal (s, vpx_codec_error(ctx));
- //   if(detail)
-   //     printf("    %s\n",detail);
 }
 
 
@@ -173,10 +251,19 @@ void setMovieViewport()
 	}
 
 	glViewport (vpOffsetX, vpOffsetY, vpWidth, vpHeight);
-	glMatrixMode(GL_PROJECTION);
-	glLoadIdentity();
-	glOrtho(0, 640, 400, 0, 1.0, -1.0);
-	glMatrixMode(GL_MODELVIEW);
+
+	const GLfloat bPMVMatrix[] =
+	{
+	2.0f/640.f,          .0,   .0,  .0,
+	        .0, -2.0f/400.f,   .0,  .0,
+	        .0,          .0, 1.0f,  .0,
+	      -1.0,        1.0f,   .0, 1.0f
+
+	};
+	for (int i = 0; i < 16; i++)
+	{
+		aPMVMatrix[i] = bPMVMatrix[i];
+	}
 }
 
 static uint64_t xiph_lace_value(unsigned char ** np)
@@ -211,49 +298,50 @@ ALuint feedAudio (void *userdata, ALubyte *data, ALuint length) {
 	ALuint got = 0;
 	int bufLen;
 
-//	while (length>0) {
-		if (! buffer) {
-			bufSize = audio_queue_get(&audioQ, &buffer, 0);
-			bufOffset = 0;
-			if (bufSize <= 0) {
-				//movieSoundPlaying = false;
-				bufSize = 0;
-				buffer = NULL;
-				if (! got) {
-					got = audioChannels*2;
-					memset(data, 0, got);
-					fprintf (stderr, "Faking audio...\n");
-					fakeAudio = true;
-				}
-//				SDL_CondSignal(audioQ.cond);
-				return got;
-			}
-		}
-
-		fakeAudio = false;
-
-		if (length > bufSize-bufOffset)
-			bufLen = bufSize-bufOffset;
-		else
-			bufLen = length;
-
-		memcpy(data, buffer+bufOffset, bufLen);
-
-		bufOffset += bufLen;
-		length -= bufLen;
-		got += bufLen;
-
-		if (bufSize <= bufOffset) {
+	if (! buffer) {
+		bufSize = audio_queue_get(&audioQ, &buffer);
+		bufOffset = 0;
+		if (bufSize <= 0) {
+			bufSize = 0;
 			buffer = NULL;
-			delete [] buffer;
+			if (! got) {
+				got = audioChannels*2;
+				memset(data, 0, got);
+				fprintf (stderr, "Faking audio...\n");
+				fakeAudio = true;
+			}
+//				SDL_CondSignal(audioQ.cond);
+			return got;
 		}
-//	}
-//	SDL_CondSignal(audioQ.cond);
+	}
+
+	fakeAudio = false;
+
+	if (length > bufSize-bufOffset)
+		bufLen = bufSize-bufOffset;
+	else
+		bufLen = length;
+
+	memcpy(data, buffer+bufOffset, bufLen);
+
+	bufOffset += bufLen;
+	length -= bufLen;
+	got += bufLen;
+
+	if (bufSize <= bufOffset) {
+		buffer = NULL;
+		delete [] buffer;
+	}
+//	fprintf (stderr, "Sending %d bytes of audio.\n", got);
+	
 	return got;
 }
 
 int playMovie (int fileNumber)
 {
+	if (specialSettings & SPECIAL_SILENT)
+		return 0;
+
 	if (movieIsPlaying) return 0;
 
 	movieSoundPlaying = false;
@@ -291,16 +379,12 @@ int playMovie (int fileNumber)
     }
 
     //const SegmentInfo* const pSegmentInfo = pSegment->GetInfo();
-
     //const long long timeCodeScale = pSegmentInfo->GetTimeCodeScale();
     //const long long duration_ns = pSegmentInfo->GetDuration();
-
     //const char* const pTitle = pSegmentInfo->GetTitleAsUTF8();
     //const char* const pMuxingApp = pSegmentInfo->GetMuxingAppAsUTF8();
     //const char* const pWritingApp = pSegmentInfo->GetWritingAppAsUTF8();
 
-    //const double duration_sec = double(duration_ns) / 1000000000;
-
     const mkvparser::Tracks* pTracks = pSegment->GetTracks();
 
     unsigned long i = 0;
@@ -324,7 +408,6 @@ int playMovie (int fileNumber)
             continue;
 
         const long long trackType = pTrack->GetType();
-        //const long long trackNumber = pTrack->GetNumber();
         //const unsigned long long trackUid = pTrack->GetUid();
         //const char* pTrackName = pTrack->GetNameAsUTF8();
 
@@ -420,7 +503,6 @@ int playMovie (int fileNumber)
 			audioNsPerByte = (1000000000 / audioSampleRate) / (audioChannels * 2);
 			audioNsBuffered = 0;
 			audioBufferLen = audioChannels*audioSampleRate;
-			//fprintf (stderr, "audioNsPerByte: %lld\n", audioNsPerByte);
         }
     }
 
@@ -430,6 +512,8 @@ int playMovie (int fileNumber)
 	if (audioTrack < 0 )
 		fatal ("Movie error: No sound found.");
 
+	video_queue_init(&videoQ);
+
     const unsigned long clusterCount = pSegment->GetCount();
 
     if (clusterCount == 0)
@@ -437,17 +521,19 @@ int playMovie (int fileNumber)
         fatal ("Movie error: Segment has no clusters.\n");
     }
 
-	/* Initialize video codec */                                                    //
-    if(vpx_codec_dec_init(&codec, interface, NULL, 0))                    //
-        die_codec(&codec, "Failed to initialize decoder for movie.");         //
+	/* Initialize video codec */
+    if(vpx_codec_dec_init(&codec, interface, NULL, 0))
+        die_codec(&codec, "Failed to initialize decoder for movie.");
 
     unsigned char *frame = new unsigned char[256*1024];
+	if (! checkNew (frame)) return false;
 
     const mkvparser::Cluster* pCluster = pSegment->GetFirst();
 
 	setMovieViewport();
 
-	movieIsPlaying = 1;
+	movieIsPlaying = playing;
+	movieIsEnding = 0;
 
 	glDepthMask (GL_FALSE);
 	glDisable(GL_DEPTH_TEST);
@@ -473,8 +559,52 @@ int playMovie (int fileNumber)
 	int frameCount = pBlock->GetFrameCount();
 	time_ns = pBlock->GetTime(pCluster);
 
+	const GLfloat texCoords[] = { 
+		0.0, 0.0,
+		1.0, 0.0,
+		0.0, 1.0,
+		1.0, 1.0
+	};
+
+	const GLfloat vertices[] = { 
+		0.0, 0.0, 0.1,
+		640.0, 0.0, 0.1,
+		0.0, 400.0, 0.1,
+		640.0, 400.0, 0.1
+	};
+
+	const GLfloat vertices1[] = { 
+		7.0, 7.0, 0.1,
+		17.0, 7.0, 0.1,
+		7.0, 29.0, 0.1,
+		17.0, 29.0, 0.1
+	};
+
+	const GLfloat vertices2[] = { 
+		27.0, 7.0, 0.1,
+		37.0, 7.0, 0.1,
+		27.0, 29.0, 0.1,
+		37.0, 29.0, 0.1
+	};
+
+	const GLfloat vertices3[] = { 
+		5.0, 5.0, 0.1,
+		15.0, 5.0, 0.1,
+		5.0, 27.0, 0.1,
+		15.0, 27.0, 0.1
+	};
+
+	const GLfloat vertices4[] = { 
+		25.0, 5.0, 0.1,
+		35.0, 5.0, 0.1,
+		25.0, 27.0, 0.1,
+		35.0, 27.0, 0.1
+	};
+
 	int frameCounter = 0;
 
+	movieStartTick = SDL_GetTicks();
+	
 	while ( movieIsPlaying ) {
 
 		checkInput();
@@ -482,8 +612,8 @@ int playMovie (int fileNumber)
 			break;
 		handleInput ();
 
-		if (movieIsPlaying == 1) {
-			// Play the movie!
+		if (movieIsPlaying && (! movieIsEnding) && (videoQ.size < 100 || audioQ.size < 100)) {
+			// Decode a frame!
 
 			if  ((pCluster != NULL) && !pCluster->EOS())
 			{
@@ -520,18 +650,18 @@ int playMovie (int fileNumber)
 					if (! checkNew(frame)) return 0;
 				}
 /*
-
 				fprintf (stderr, "Block :%s,%s,%15lld\n",
 				 (trackType == VIDEO_TRACK) ? "V" : "A",
 				 pBlock->IsKey() ? "I" : "P",
 				 time_ns);
-*/
+ */
+
 				if (trackNum == videoTrack) {
 
 					theFrame.Read(&reader, frame);
 
 					/* Decode the frame */
-					if(vpx_codec_decode(&codec, frame, size, NULL, 0))                //
+					if(vpx_codec_decode(&codec, frame, size, NULL, 0))
 						die_codec(&codec, "Failed to decode frame");
 
 					// Let's decode an image frame!
@@ -544,6 +674,10 @@ int playMovie (int fileNumber)
 
 						unsigned int y;
 
+						GLubyte * ytex=NULL;
+						GLubyte * utex=NULL;
+						GLubyte * vtex=NULL;
+						
 						if (! ytex) {
 							ytex = new GLubyte [img->d_w * img->d_h];
 							utex = new GLubyte [(img->d_w >> 1) * (img->d_h >> 1)];
@@ -551,25 +685,6 @@ int playMovie (int fileNumber)
 							if (!ytex || !utex || !vtex)
 								fatal (ERROR_OUT_OF_MEMORY);
 
-							glEnable (GL_TEXTURE_2D);
-							if (! yTextureName) glGenTextures (1, &yTextureName);
-							if (! uTextureName) glGenTextures (1, &uTextureName);
-							if (! vTextureName) glGenTextures (1, &vTextureName);
-							glBindTexture (GL_TEXTURE_2D, yTextureName);
-							glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, img->d_w, img->d_h, 0,
-										 GL_ALPHA, GL_UNSIGNED_BYTE, ytex);
-							glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-							glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-							glBindTexture (GL_TEXTURE_2D, uTextureName);
-							glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, img->d_w>>1, img->d_h>>1, 0,
-										 GL_ALPHA, GL_UNSIGNED_BYTE, utex);
-							glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-							glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-							glBindTexture (GL_TEXTURE_2D, vTextureName);
-							glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, img->d_w>>1, img->d_h>>1, 0,
-										 GL_ALPHA, GL_UNSIGNED_BYTE, vtex);
-							glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-							glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 						}
 
 						unsigned char *buf =img->planes[0];
@@ -587,81 +702,11 @@ int playMovie (int fileNumber)
 							memcpy(vtex+y*(img->d_w>>1), buf, img->d_w>>1);
 							buf += img->stride[2];
 						}
-						glBindTexture (GL_TEXTURE_2D, yTextureName);
-						glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, img->d_w, img->d_h,
-										GL_ALPHA, GL_UNSIGNED_BYTE, ytex);
-						glBindTexture (GL_TEXTURE_2D, uTextureName);
-						glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, img->d_w>>1, img->d_h>>1,
-										GL_ALPHA, GL_UNSIGNED_BYTE, utex);
-						glBindTexture (GL_TEXTURE_2D, vTextureName);
-						glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, img->d_w>>1, img->d_h>>1,
-										GL_ALPHA, GL_UNSIGNED_BYTE, vtex);
+						
+						video_queue_put(&videoQ, ytex, utex, vtex,
+										img->d_w, img->d_h, time_ns/1000000);
+						
 
-						if (movieSoundPlaying) {
-							audioBuffers *x;
-							while (movieIsPlaying == 1 && ! fakeAudio && movieSoundPlaying && getSoundSource(movieAudioIndex)) {
-								//SDL_LockMutex(audioQ.mutex);
-								if (! (x = audioQ.first))
-									break;
-								if (! (x = x->next))
-									break;
-								if (! (x = x->next))
-									break;
-								if (! (x = x->next))
-									break;
-								if (! (x = x->next))
-									break;
-								if (! (x = x->next))
-									break;
-								if (! (x = x->next))
-									break;
-								if (! (x = x->next))
-									break;
-								if (! (x = x->next))
-									break;
-								//SDL_CondWait(audioQ.cond, audioQ.mutex);
-								//SDL_UnlockMutex(audioQ.mutex);
-								//SDL_Delay(1);
-							}
-							#if 0
-							ALuint source = getSoundSource(movieAudioIndex);
-							ALint queued, offset, state;
-							if (source) {
-								while (movieIsPlaying == 1) {
-									audioNsPlayed = audioNsBuffered;
-									EnterCriticalSection(&cs_StreamPlay);
-									alGetSourcei(source, AL_BUFFERS_QUEUED, &queued);
-									/* Order here is important. If the offset is queried after the state and an
-									 * underrun occurs in between the two calls, it can end up reporting a 0
-									 * offset in a playing state. */
-									//alGetSourcei(source, AL_BYTE_OFFSET, &offset);
-									alGetSourcei(source, AL_SOURCE_STATE, &state);
-
-									/* Note: state=stopped is an underrun, meaning all buffers are processed
-									 * and the whole queue has been played. */
-									if(alGetError() == AL_NO_ERROR && state != AL_STOPPED)
-									{
-										//audioNsPlayed -= queued*chunk_length;
-										audioNsPlayed -= queued*512*audioNsPerByte;
-										//audioNsPlayed += offset*audioNsPerByte;
-									} else {
-										audioNsPlayed = time_ns;
-									}
-
-									LeaveCriticalSection(&cs_StreamPlay);
-
-									if (time_ns<=audioNsPlayed)
-										break;
-									/*
-									checkInput();
-									if (weAreDoneSoQuit)
-										break;
-									handleInput ();*/
-									//SDL_Delay(1);
-								}
-							}
-							#endif
-						}
 					}
 				} else if (trackNum == audioTrack) {
 					// Use this Audio Track
@@ -692,6 +737,7 @@ int playMovie (int fileNumber)
 							vorbis_fpu_control fpu;
 
 							char * buffer = new char[bytespersample*numSamples];
+							if (! checkNew (buffer)) return false;
 
 							/* a tight loop to pack each size */
 							{
@@ -749,10 +795,9 @@ int playMovie (int fileNumber)
 
 							vorbis_synthesis_read(&vorbisDspState,numSamples);
 							audioBufferLen = bytespersample*numSamples;
-							audio_queue_put(&audioQ, buffer, audioBufferLen);
+							audio_queue_put(&audioQ, buffer, audioBufferLen, time_ns/1000000);
 
-							//audioNsBuffered = time_ns + audioNsPerByte*audioBufferLen;
-						//	fprintf (stderr, "Audio buffered: %lld\n", audioBufferLen);
+							//fprintf (stderr, "Audio buffered: %lld byte %lld ns\n",audioBufferLen, audioNsPerByte*audioBufferLen);
 
 							if (! movieSoundPlaying && size > 1) {
 								fprintf (stderr, "** starting sound ** \n");
@@ -761,21 +806,80 @@ int playMovie (int fileNumber)
 							}
 						}
 					}
-
-
 				}
 				++frameCounter;
 
-
 			} else {
-movieHasEnded:	movieIsPlaying = 0;
+movieHasEnded:	movieIsEnding = 1;
 			}
-
-
 		}
 
-		// Don't update the screen on audio frames
-		if (trackNum == videoTrack || movieIsPlaying == 2) {
+		bool videoUpdated = false;
+		// Get a video frame.
+		if (movieIsPlaying == playing) {
+			
+			videoBuffers *vB;
+			// Do we have decoded video waiting?
+			if (vB = videoQ.first) {
+				Uint32 tick = SDL_GetTicks() - movieStartTick;
+				
+				// Is it time to display the frame yet?
+				if ((tick+1) < vB->time_ms) {
+					SDL_Delay(1);
+				} else {
+					GLubyte * ytex=NULL;
+					GLubyte * utex=NULL;
+					GLubyte * vtex=NULL;
+					GLsizei w, h;
+						
+					if (video_queue_get(&videoQ, &ytex, &utex, &vtex, &w, &h)) {
+						
+						if (! yTextureName) glGenTextures (1, &yTextureName);
+						if (! uTextureName) glGenTextures (1, &uTextureName);
+						if (! vTextureName) glGenTextures (1, &vTextureName);
+						glBindTexture (GL_TEXTURE_2D, yTextureName);
+						glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, w, h, 0,
+										 GL_ALPHA, GL_UNSIGNED_BYTE, ytex);
+						glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+						glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+						glBindTexture (GL_TEXTURE_2D, uTextureName);
+						glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, w>>1, h>>1, 0,
+										 GL_ALPHA, GL_UNSIGNED_BYTE, utex);
+						glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+						glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+						glBindTexture (GL_TEXTURE_2D, vTextureName);
+						glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, w>>1, h>>1, 0,
+										 GL_ALPHA, GL_UNSIGNED_BYTE, vtex);
+						glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+						glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+
+						
+						glBindTexture (GL_TEXTURE_2D, yTextureName);
+						glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, w, h,
+										GL_ALPHA, GL_UNSIGNED_BYTE, ytex);
+						glBindTexture (GL_TEXTURE_2D, uTextureName);
+						glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, w>>1, h>>1,
+											GL_ALPHA, GL_UNSIGNED_BYTE, utex);
+						glBindTexture (GL_TEXTURE_2D, vTextureName);
+						glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, w>>1, h>>1,
+											GL_ALPHA, GL_UNSIGNED_BYTE, vtex);
+														
+						delete [] ytex;
+						delete [] utex;
+						delete [] vtex;
+						ytex = utex = vtex = NULL;
+						videoUpdated = true;
+							
+					}
+				}
+			} else if (movieIsEnding) {
+				// We have reached the end of the movie.
+				movieIsPlaying = nothing;
+			}
+		}
+			
+		// Update the screen if there's new video, or if we're paused
+		if (videoUpdated || movieIsPlaying == paused) {
 
 			// Clear The Screen
 			glClear(GL_COLOR_BUFFER_BIT);
@@ -784,100 +888,88 @@ movieHasEnded:	movieIsPlaying = 0;
 			if (shader.yuv) {
 				glUseProgram(shader.yuv);
 				glActiveTexture(GL_TEXTURE1);
-				glEnable(GL_TEXTURE_2D);
 				glBindTexture (GL_TEXTURE_2D, uTextureName);
 				glActiveTexture(GL_TEXTURE2);
-				glEnable(GL_TEXTURE_2D);
 				glBindTexture (GL_TEXTURE_2D, vTextureName);
 				glActiveTexture(GL_TEXTURE0);
 			}
-			glEnable (GL_TEXTURE_2D);
 			glBindTexture (GL_TEXTURE_2D, yTextureName);
 			glEnable(GL_BLEND);
-			glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-			glColor4f(1.0, 1.0, 1.0, 1.0);
+			//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 
-			glBegin(GL_QUADS);
-
-			glTexCoord2f(0.0, 0.0); glVertex3f(0.0, 0.0, 0.1);
-			glTexCoord2f(1.0, 0.0); glVertex3f(640.0, 0.0, 0.1);
-			glTexCoord2f(1.0, 1.0); glVertex3f(640.0, 400.0, 0.1);
-			glTexCoord2f(0.0, 1.0); glVertex3f(0.0, 400.0, 0.1);
-
-			glEnd();
-
-			glActiveTexture(GL_TEXTURE1);
-			glDisable(GL_TEXTURE_2D);
-			glActiveTexture(GL_TEXTURE2);
-			glDisable(GL_TEXTURE_2D);
-			glActiveTexture(GL_TEXTURE0);
+			setPMVMatrix(shader.yuv);
+			drawQuad(shader.yuv, vertices, 1, texCoords);
 
 			glUseProgram(0);
 
-			if (movieIsPlaying == 2) {
+			if (movieIsPlaying == paused) {
 				pausefade -= 1.0 / 24;
 				if (pausefade<-1.0) pausefade = 1.0;
 
 				// Paused.
-				glDisable (GL_TEXTURE_2D);
 				glEnable(GL_BLEND);
-				glColor4f(0.0, 0.0, 0.0, fabs(pausefade));
 
-				glBegin(GL_QUADS);
+				glUseProgram(shader.color);
+				setPMVMatrix(shader.color);
+				setPrimaryColor(0.0f, 0.0f, 0.0f, fabs(pausefade));
+				drawQuad(shader.color, vertices1, 0);
+				drawQuad(shader.color, vertices2, 0);
+				setPrimaryColor(1.0f, 1.0f, 1.0f, fabs(pausefade));
+				drawQuad(shader.color, vertices3, 0);
+				drawQuad(shader.color, vertices4, 0);
+				glUseProgram(0);
 
 
-				glVertex3f(7.0, 7.0, 0.1);
-				glVertex3f(17.0, 7.0, 0.1);
-				glVertex3f(17.0, 29.0, 0.1);
-				glVertex3f(7.0, 29.0, 0.1);
-
-				glVertex3f(27.0, 7.0, 0.1);
-				glVertex3f(37.0, 7.0, 0.1);
-				glVertex3f(37.0, 29.0, 0.1);
-				glVertex3f(27.0, 29.0, 0.1);
-
-				glColor4f(1.0, 1.0, 1.0, fabs(pausefade));
-
-				glVertex3f(5.0, 5.0, 0.1);
-				glVertex3f(15.0, 5.0, 0.1);
-				glVertex3f(15.0, 27.0, 0.1);
-				glVertex3f(5.0, 27.0, 0.1);
-
-				glVertex3f(25.0, 5.0, 0.1);
-				glVertex3f(35.0, 5.0, 0.1);
-				glVertex3f(35.0, 27.0, 0.1);
-				glVertex3f(25.0, 27.0, 0.1);
-
-				glEnd();
-
 				glDisable(GL_BLEND);
 
 				Wait_Frame();
 			}
 			glFlush();
-			SDL_GL_SwapBuffers();
+#if !defined(HAVE_GLES2)
+        		SDL_GL_SwapBuffers();
+#else
+			EGL_SwapBuffers();
+#endif
 
 		}
+		videoUpdated = false;
 	}
 
 	// Cleanup
+	movieIsPlaying = nothing;
 	for (int i =0; i<10; i++) Wait_Frame();
 	huntKillFreeSound(fileNumber);
-    if(vpx_codec_destroy(&codec))                                             //
-        die_codec(&codec, "Failed to destroy codec");                         //
+    if(vpx_codec_destroy(&codec))
+        die_codec(&codec, "Failed to destroy codec");
 	vorbis_dsp_clear(&vorbisDspState);
 	vorbis_block_clear(&vorbisBlock);
 	vorbis_comment_clear(&vorbisComment);
 	vorbis_info_clear(&vorbisInfo);
     delete pSegment;
-	delete [] ytex;
-	delete [] utex;
-	delete [] vtex;
-	ytex = utex = vtex = NULL;
-	glDeleteTextures (1, &yTextureName);
-	glDeleteTextures (1, &uTextureName);
-	glDeleteTextures (1, &vTextureName);
-	movieIsPlaying = 0;
+	deleteTextures (1, &yTextureName);
+	deleteTextures (1, &uTextureName);
+	deleteTextures (1, &vTextureName);
+	yTextureName = uTextureName = vTextureName = 0;
+	
+	// Delete any remaining buffers
+	videoBuffers *vB = videoQ.first;
+	while (vB = videoQ.first) {
+		videoQ.first = vB->next;
+		delete [] vB->ytex;
+		delete [] vB->utex;
+		delete [] vB->vtex;		
+		delete vB;
+	}
+	videoQ.size = 0;
+
+	audioBuffers *aB = audioQ.first;
+	while (aB = audioQ.first) {
+		audioQ.first = aB->next;
+		delete [] aB->buffer;
+		delete aB;
+	}
+	audioQ.size = 0;
+	
 	Init_Timer();
 	glViewport (viewportOffsetX, viewportOffsetY, viewportWidth, viewportHeight);
 	setPixelCoords (false);
@@ -887,13 +979,13 @@ movieHasEnded:	movieIsPlaying = 0;
 int stopMovie ()
 {
 	int r = movieIsPlaying;
-	movieIsPlaying = 0;
+	movieIsPlaying = nothing;
 	return r;
 }
 
 int pauseMovie()
-{
-	if (movieIsPlaying == 1) {
+{	
+	if (movieIsPlaying == playing) {
 		ALuint source = getSoundSource(movieAudioIndex);
 		if (source) {
 #if defined __unix__
@@ -902,9 +994,9 @@ int pauseMovie()
 			alurePauseSource(source, false);
 #endif
 		}
-		movieIsPlaying = 2;
+		movieIsPlaying = paused;
 		fprintf (stderr, "** Pausing **\n");
-	} else if (movieIsPlaying == 2) {
+	} else if (movieIsPlaying == paused) {
 		ALuint source = getSoundSource(movieAudioIndex);
 		if (source) {
 #if defined __unix__
@@ -914,7 +1006,7 @@ int pauseMovie()
 #endif
 		}
 		fprintf (stderr, "** Restarted movie ** sound: %d source: %d\n", movieSoundPlaying, source);
-		movieIsPlaying = 1;
+		movieIsPlaying = playing;
 	}
 	return movieIsPlaying;
 }
