@@ -85,6 +85,8 @@ personaAnimation * createPersonaAnim (int num, variableStack * & stacky) {
 
 personaAnimation * makeNullAnim () {
 	personaAnimation * newAnim	= new personaAnimation;
+	if (! checkNew (newAnim)) return false;
+
 
 	newAnim -> theSprites		= NULL;
 	newAnim -> numFrames		= 0;
@@ -96,6 +98,7 @@ personaAnimation * copyAnim (personaAnimation * orig) {
 	int num = orig -> numFrames;
 
 	personaAnimation * newAnim	= new personaAnimation;
+	if (! checkNew (newAnim)) return false;
 
 	// Copy the easy bits...
 	newAnim -> theSprites		= orig -> theSprites;
@@ -105,7 +108,8 @@ personaAnimation * copyAnim (personaAnimation * orig) {
 
 		// Argh! Frames! We need a whole NEW array of animFrame structures...
 
-		newAnim -> frames = new animFrame[num];
+		newAnim->frames = new animFrame[num];
+		if (! checkNew (newAnim->frames)) return false;
 
 		for (int a = 0; a < num; a ++) {
 			newAnim -> frames[a].frameNum = orig -> frames[a].frameNum;
