@@ -8,11 +8,12 @@
  */
 
 
+#if !defined(HAVE_GLES2)
 #include "GLee.h"
+#else
+#include <GLES2/gl2.h>
+#endif
 
 char *shaderFileRead(const char *fn); 
-int buildShaders (const GLchar *vertexShader, const GLchar *fragmentShader);
-
-int printOglError (const char *file, int         line);
-#define printOpenGLError() printOglError(__FILE__, __LINE__)
+int buildShaders (const char *vertexShader, const char *fragmentShader);
 
