@@ -11,7 +11,12 @@
 #include <sys/stat.h>
 #include <iconv.h>
 
+#if !defined(HAVE_GLES2)
 #include "GLee.h"
+#else
+#include <GLES2/gl2.h>
+#include "eglport/eglport.h"
+#endif
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -170,7 +175,6 @@ FILE * openAndVerify (char * filename, char extra1, char extra2, const char * er
 bool initSludge (char * filename) {
 	int a = 0;
 	mouseCursorAnim = makeNullAnim ();
-	createFontPalette (pastePalette);
 
 	FILE * fp = openAndVerify (filename, 'G', 'E', ERROR_BAD_HEADER, gameVersion);
 	if (! fp) return false;
@@ -178,17 +182,23 @@ bool initSludge (char * filename) {
 	if (fgetc (fp)) {
 		numBIFNames = get2bytes (fp);
 		allBIFNames = new char * [numBIFNames];
+		if (! checkNew (allBIFNames)) return false;
+
 		for (int fn = 0; fn < numBIFNames; fn ++) {
 			allBIFNames[fn] = readString (fp);
 		}
 		numUserFunc = get2bytes (fp);
 		allUserFunc = new char * [numUserFunc];
+		if (! checkNew (allUserFunc)) return false;
+
 		for (int fn = 0; fn < numUserFunc; fn ++) {
 			allUserFunc[fn] = readString (fp);
 		}
 		if (gameVersion >= VERSION(1,3)) {
 			numResourceNames = get2bytes (fp);
 			allResourceNames = new char * [numResourceNames];
+			if (! checkNew (allResourceNames)) return false;
+
 			for (int fn = 0; fn < numResourceNames; fn ++) {
 				allResourceNames[fn] = readString (fp);
 			}
@@ -467,26 +477,10 @@ bool initSludge (char * filename) {
 	// Get the original (untranslated) name of the game and convert it to Unicode.
 	// We use this to find saved preferences and saved games.
 	setFileIndices (fp, gameSettings.numLanguages, 0);
-	char * gameNameWin = getNumberedString(1);
-	char * gameName = new char[ 1024];
-	char **tmp1 = &gameNameWin;
-	char **tmp2 = &gameName;
-	char * nameOrig = gameNameWin;
-	char * gameNameOrig = gameName;
+	char * gameNameOrig = getNumberedString(1);
 
-	iconv_t convert = iconv_open ("UTF-8", "CP1252");
-	size_t len1 = strlen(gameNameWin)+1;
-	size_t len2 = 1023;
-#ifdef WIN32
-	iconv (convert,(const char **) tmp1, &len1, tmp2, &len2);
-#else
-	iconv (convert,(char **) tmp1, &len1, tmp2, &len2);
-#endif
-	iconv_close (convert);
+	char * gameName = encodeFilename (gameNameOrig);
 
-	gameName = encodeFilename (gameNameOrig);
-
-	delete nameOrig;
 	delete gameNameOrig;
 
 	changeToUserDir ();
@@ -499,13 +493,22 @@ bool initSludge (char * filename) {
 
 	if (chdir (gameName)) return fatal ("This game's preference folder is inaccessible!\nI can't access the following directory (maybe there's a file with the same name, or maybe it's read-protected):", gameName);
 
+	delete [] gameName;
+	
 	// Get user settings
 	readIniFile (filename);
 
+	// There's no startup window on Linux and respecting this 
+	// option from the ini file would disable commandline options.
+#if defined __unix__ && !(defined __APPLE__)
+		if (! showSetupWindow()) return 0;
+		saveIniFile (filename);
+#else
 	if (! gameSettings.noStartWindow) {
 		if (! showSetupWindow()) return 0;
 		saveIniFile (filename);
 	}
+#endif
 
 	// Now set file indices properly to the chosen language.
 	languageNum = getLanguageForFileB ();
@@ -534,12 +537,12 @@ extern int cameraX, cameraY;
 extern float cameraZoom;
 
 bool checkColourChange (bool reset) {
-	static GLuint oldPixel;
-	static GLuint pixel;
+	static GLushort oldPixel;
+	static GLushort pixel;
 
 	glReadPixels((GLint)(viewportOffsetX+input.mouseX*viewportWidth/((float)winWidth/cameraZoom)),
         (GLint)(viewportOffsetY+(((float)winHeight/cameraZoom) - input.mouseY)*viewportHeight/((float)winHeight/cameraZoom)),
-        1, 1, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8, &pixel);
+        1, 1, GL_RGBA, GL_UNSIGNED_SHORT_4_4_4_4, &pixel);
 
 	if (reset || oldPixel != pixel) {
 		oldPixel = pixel;
@@ -568,8 +571,11 @@ void sludgeDisplay () {
 	if (brightnessLevel < 255) fixBrightness ();	// This is for transitionLevel special effects
 
 	glFlush();
-	SDL_GL_SwapBuffers();
-
+#if !defined(HAVE_GLES2)
+        SDL_GL_SwapBuffers();
+#else
+	EGL_SwapBuffers();
+#endif
 }
 
 void pauseFunction (loadedFunction * fun) {
@@ -742,7 +748,6 @@ bool continueFunction (loadedFunction * fun) {
 				case SVT_BUILT:
 					{
 					builtReturn br = callBuiltIn (fun -> reg.varData.intValue, param, fun);
-				    //fprintf (stderr, "Function returned. \n");    fflush (stderr);
 
 					switch (br) {
 						case BR_ERROR:
@@ -1255,8 +1260,8 @@ bool handleInput () {
 
 	if (lastRegion != overRegion && currentEvents -> focusFunction) {
 		variableStack * tempStack = new variableStack;
-
 		if (! checkNew (tempStack)) return false;
+		
 		initVarNew (tempStack -> thisVar);
 		if (overRegion) {
 			setVariable (tempStack -> thisVar, SVT_OBJTYPE, overRegion -> thisType -> objectNum);
@@ -1280,7 +1285,8 @@ bool handleInput () {
 	if (input.keyPressed && currentEvents -> spaceFunction) {
 		char * tempString = NULL;
 		switch (input.keyPressed) {
-			case 127:	tempString = copyString ("BACKSPACE");	break;
+			case 127:	
+				tempString = copyString ("BACKSPACE");	break;
 			case 9:		tempString = copyString ("TAB");		break;
 			case 13:	tempString = copyString ("ENTER");		break;
 			case 27:	tempString = copyString ("ESCAPE");		break;
