@@ -31,25 +31,60 @@ unsigned char currentBurnR = 0, currentBurnG = 0, currentBurnB = 0;
 
 
 void forgetSpriteBank (spriteBank & forgetme) {
-	glDeleteTextures (forgetme.myPalette.numTextures, forgetme.myPalette.tex_names);
-	if (forgetme.isFont) glDeleteTextures (forgetme.myPalette.numTextures, forgetme.myPalette.burnTex_names);
+
+	deleteTextures (forgetme.myPalette.numTextures, forgetme.myPalette.tex_names);
+	if (forgetme.isFont) {
+		deleteTextures (forgetme.myPalette.numTextures, forgetme.myPalette.burnTex_names);
+		delete [] forgetme.myPalette.burnTex_names;
+		forgetme.myPalette.burnTex_names = NULL;
+	}
+	
+	delete [] forgetme.myPalette.tex_names;
+	forgetme.myPalette.tex_names = NULL;
+	delete [] forgetme.myPalette.tex_w;
+	forgetme.myPalette.tex_w = NULL;
+	delete [] forgetme.myPalette.tex_h;
+	forgetme.myPalette.tex_h = NULL;
+	
+	if (forgetme.myPalette.pal) {
+		delete  [] forgetme.myPalette.pal;
+		forgetme.myPalette.pal = NULL;
+		delete  [] forgetme.myPalette.r;
+		forgetme.myPalette.r = NULL;
+		delete  [] forgetme.myPalette.g;
+		forgetme.myPalette.g = NULL;
+		delete  [] forgetme.myPalette.b;
+		forgetme.myPalette.b = NULL;
+	}
 
 	delete forgetme.sprites;
 	forgetme.sprites = NULL;
 }
 
 bool reserveSpritePal (spritePalette & sP, int n) {
+	if (sP.pal) {
+		delete  [] sP.pal;
+		delete  [] sP.r;
+		delete  [] sP.g;
+		delete  [] sP.b;
+	}
+
 	sP.pal = new unsigned short int [n];
+	if (! checkNew (sP.pal)) return false;
+
 	sP.r = new unsigned char [n];
+	if (! checkNew (sP.r)) return false;
 	sP.g = new unsigned char [n];
+	if (! checkNew (sP.g)) return false;
 	sP.b = new unsigned char [n];
+	if (! checkNew (sP.b)) return false;
 	sP.total = n;
 	return (bool) (sP.pal != NULL) && (sP.r != NULL) && (sP.g != NULL) && (sP.b != NULL);
 }
 
 bool loadSpriteBank (int fileNum, spriteBank & loadhere, bool isFont) {
 	int i, tex_num, total, picwidth, picheight, spriteBankVersion = 0, howmany=0, startIndex=0;
-	int totalwidth[256], maxheight[256];
+	int * totalwidth, * maxheight;
 	int numTextures = 0;
 	byte * data;
 
@@ -73,9 +108,28 @@ bool loadSpriteBank (int fileNum, spriteBank & loadhere, bool isFont) {
 
 	loadhere.total = total;
 	loadhere.sprites = new sprite [total];
-	byte ** spriteData = new byte * [total];
 	if (! checkNew (loadhere.sprites)) return false;
+	byte ** spriteData = new byte * [total];
+	if (! checkNew (spriteData)) return false;
 
+	totalwidth = new int[total];
+	if (! checkNew (totalwidth)) return false;
+
+	maxheight = new int[total];
+	if (! checkNew (maxheight)) return false;
+
+	loadhere.myPalette.tex_names = new GLuint [total];
+	if (! checkNew (loadhere.myPalette.tex_names)) return false;
+
+	if (isFont) {
+		loadhere.myPalette.burnTex_names = new GLuint [total];
+		if (! checkNew (loadhere.myPalette.burnTex_names)) return false;
+	}
+	loadhere.myPalette.tex_w = new int [total];
+	if (! checkNew (loadhere.myPalette.tex_w)) return false;
+	loadhere.myPalette.tex_h = new int [total];
+	if (! checkNew (loadhere.myPalette.tex_h)) return false;
+	
 	if (spriteBankVersion && spriteBankVersion < 3) {
 		howmany = fgetc (bigDataFile);
 		startIndex = 1;
@@ -119,6 +173,8 @@ bool loadSpriteBank (int fileNum, spriteBank & loadhere, bool isFont) {
 
 				unsigned char * row_pointers[height];
 				spriteData[i] = new unsigned char [rowbytes*height];
+				if (! checkNew (spriteData[i])) return false;
+
 				for (unsigned int row = 0; row<height; row++)
 					row_pointers[row] = spriteData[i] + row*rowbytes;
 
@@ -227,9 +283,11 @@ bool loadSpriteBank (int fileNum, spriteBank & loadhere, bool isFont) {
 			maxheight[tex_num] = getNextPOT(maxheight[tex_num]);
 		}
 		tmp[tex_num] = new GLubyte [(maxheight[tex_num]+1)*totalwidth[tex_num]*4];
+		if (! checkNew (tmp[tex_num])) return false;
 		memset (tmp[tex_num], 0, maxheight[tex_num]*totalwidth[tex_num]*4);
 		if (isFont) {
 			tmp2[tex_num] = new GLubyte [(maxheight[tex_num]+1)*totalwidth[tex_num]*4];
+			if (! checkNew (tmp2[tex_num])) return false;
 			memset (tmp2[tex_num], 0, maxheight[tex_num]*totalwidth[tex_num]*4);
 		}
 		loadhere.myPalette.tex_w[tex_num] = totalwidth[tex_num];
@@ -315,7 +373,7 @@ bool loadSpriteBank (int fileNum, spriteBank & loadhere, bool isFont) {
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 		}
-		glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, totalwidth[tex_num], maxheight[tex_num], 0, GL_RGBA, GL_UNSIGNED_BYTE, tmp[tex_num]);
+		texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, totalwidth[tex_num], maxheight[tex_num], 0, GL_RGBA, GL_UNSIGNED_BYTE, tmp[tex_num], loadhere.myPalette.tex_names[tex_num]);
 
 		delete tmp[tex_num];
 		tmp[tex_num] = NULL;
@@ -331,7 +389,7 @@ bool loadSpriteBank (int fileNum, spriteBank & loadhere, bool isFont) {
 				glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 				glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 			}
-			glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, totalwidth[tex_num], maxheight[tex_num], 0, GL_RGBA, GL_UNSIGNED_BYTE, tmp2[tex_num]);
+			texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, totalwidth[tex_num], maxheight[tex_num], 0, GL_RGBA, GL_UNSIGNED_BYTE, tmp2[tex_num], loadhere.myPalette.burnTex_names[tex_num]);
 
 			delete tmp2[tex_num];
 			tmp2[tex_num] = NULL;
@@ -377,8 +435,8 @@ void pasteSpriteToBackDrop (int x1, int y1, sprite & single, const spritePalette
 	const GLfloat btexCoords[] = { 
 		btx1, bty1,
 		btx2, bty1,
-		btx2, bty2, 
-		btx1, bty2
+		btx1, bty2,
+		btx2, bty2
 	}; 
 
 	if (x1 < 0) diffX += x1;
@@ -400,56 +458,40 @@ void pasteSpriteToBackDrop (int x1, int y1, sprite & single, const spritePalette
 
 			// Render the sprite to the backdrop
 			// (using mulitexturing, so the backdrop is seen where alpha < 1.0)
-			glClientActiveTexture(GL_TEXTURE2);
 			glActiveTexture(GL_TEXTURE2);
 			glBindTexture (GL_TEXTURE_2D, backdropTextureName);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-			glTexCoordPointer(2, GL_FLOAT, 0, btexCoords);
-			glClientActiveTexture(GL_TEXTURE0);
 			glActiveTexture(GL_TEXTURE0);
 
 			glUseProgram(shader.paste);
 			GLint uniform = glGetUniformLocation(shader.paste, "useLightTexture");
 			if (uniform >= 0) glUniform1i(uniform, 0); // No lighting
 
-			glColor4ub (fontPal.originalRed, fontPal.originalGreen, fontPal.originalBlue, 255);
-			glBindTexture (GL_TEXTURE_2D, fontPal.tex_names[single.texNum]);
-			glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+			setPMVMatrix(shader.paste);
 
-			const GLint vertices[] = { 
-				-xoffset, -yoffset, 0,
-				single.width-xoffset, -yoffset, 0,
-				single.width-xoffset, single.height-yoffset, 0,
-				-xoffset, single.height-yoffset, 0
+			setPrimaryColor(fontPal.originalRed/255.f, fontPal.originalGreen/255.f, fontPal.originalBlue/255.f, 1.0f);
+			glBindTexture (GL_TEXTURE_2D, fontPal.tex_names[single.texNum]);
+			//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+
+			const GLfloat vertices[] = { 
+				(GLfloat)-xoffset, (GLfloat)-yoffset, 0.,
+				(GLfloat)single.width-xoffset, (GLfloat)-yoffset, 0.,
+				(GLfloat)-xoffset, (GLfloat)single.height-yoffset, 0.,
+				(GLfloat)single.width-xoffset, (GLfloat)single.height-yoffset, 0.
 			};
 
 			const GLfloat texCoords[] = { 
 				tx1, ty1,
 				tx2, ty1,
-				tx2, ty2, 
-				tx1, ty2
+				tx1, ty2,
+				tx2, ty2
 			}; 
 
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-			glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
-			glVertexPointer(3, GL_INT, 0, vertices);
-
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
-
-			glClientActiveTexture(GL_TEXTURE2);
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glClientActiveTexture(GL_TEXTURE0);
+			drawQuad(shader.paste, vertices, 3, texCoords, NULL, btexCoords);
 
 			// Copy Our ViewPort To The Texture
-			glBindTexture(GL_TEXTURE_2D, backdropTextureName);
 			glUseProgram(0);
 
-			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, (int) ((x1<0) ? xoffset: x1+xoffset), (int) ((y1<0) ? yoffset: y1+yoffset), (int) ((x1<0) ?viewportOffsetX-x1:viewportOffsetX), (int) ((y1<0) ?viewportOffsetY-y1:viewportOffsetY), w, h);
+			copyTexSubImage2D(GL_TEXTURE_2D, 0, (int) ((x1<0) ? xoffset: x1+xoffset), (int) ((y1<0) ? yoffset: y1+yoffset), (int) ((x1<0) ?viewportOffsetX-x1:viewportOffsetX), (int) ((y1<0) ?viewportOffsetY-y1:viewportOffsetY), w, h, backdropTextureName);
 
 			yoffset += viewportHeight;
 		}
@@ -465,26 +507,38 @@ void burnSpriteToBackDrop (int x1, int y1, sprite & single, const spritePalette
 	float tx2 = (float)(single.tex_x + single.width + 0.5) / fontPal.tex_w[single.texNum];
 	float ty2 = (float)(single.height+2)/fontPal.tex_h[single.texNum];
 
-	const GLfloat backdropTexCoords[] = { 
-		0.0f, 0.0f,
-		backdropTexW, 0.0f,
-		backdropTexW, backdropTexH, 
-		0.0f, backdropTexH
-	}; 
-
 	const GLfloat spriteTexCoords[] = { 
 		tx1, ty1,
 		tx2, ty1,
-		tx2, ty2, 
-		tx1, ty2
+		tx1, ty2,
+		tx2, ty2
+	}; 
+
+	x1 -= single.xhot;
+	y1 -= single.yhot-1;
+
+	float bx1 = (float)x1*backdropTexW/sceneWidth;
+	float by1 = (float)y1*backdropTexH/sceneHeight;
+	float bx2 = (float)(x1+single.width-1)*backdropTexW/sceneWidth;
+	float by2 = (float)(y1+single.height-1)*backdropTexH/sceneHeight;
+
+	const GLfloat backdropTexCoords[] = { 
+		bx1, by1,
+		bx2, by1,
+		bx1, by2,
+		bx2, by2
+	};
+
+	const GLfloat backdropTexCoords2[] = { 
+		0.0f, 0.0f,
+		backdropTexW, 0.0f,
+		0.0f, backdropTexH,
+		backdropTexW, backdropTexH
 	}; 
 
 	int diffX = single.width+1;
 	int diffY = single.height+2;
 
-	x1 -= single.xhot;
-	y1 -= single.yhot-1;
-
 	if (x1 < 0) diffX += x1;
 	if (y1 < 0) diffY += y1;
 	if (x1 + diffX > sceneWidth) diffX = sceneWidth - x1;
@@ -493,7 +547,7 @@ void burnSpriteToBackDrop (int x1, int y1, sprite & single, const spritePalette
 	if (diffY < 0) return;
 
 	setPixelCoords (true);
-	glColor3ub (currentBurnR, currentBurnG, currentBurnB);
+	setPrimaryColor(currentBurnR/255.f, currentBurnG/255.f, currentBurnB/255.f, 1.0f);
 
 	GLfloat xoffset = 0.0f;
 	while (xoffset < diffX) {
@@ -503,59 +557,45 @@ void burnSpriteToBackDrop (int x1, int y1, sprite & single, const spritePalette
 		while (yoffset < diffY) {
 			int h = (diffY-yoffset < viewportHeight) ? diffY-yoffset : viewportHeight;
 
-			// Render the scene - first the old backdrop
-			glBindTexture (GL_TEXTURE_2D, backdropTextureName);
-			glBindTexture (GL_TEXTURE_2D, backdropTextureName);
-			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-			glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
-
 			const GLfloat backdropVertices[] = { 
 				-x1-xoffset, -y1+yoffset, 0.0f, 
 				sceneWidth-1.0f-x1-xoffset, -y1+yoffset, 0.0f, 
-				sceneWidth-1.0f-x1-xoffset, sceneHeight-1.0f-y1+yoffset, 0.0f, 
-				-x1-xoffset, sceneHeight-1.0f-y1+yoffset, 0.0f
+				-x1-xoffset, sceneHeight-1.0f-y1+yoffset, 0.0f,
+				sceneWidth-1.0f-x1-xoffset, sceneHeight-1.0f-y1+yoffset, 0.0f
 			};
 
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-			glVertexPointer(3, GL_FLOAT, 0, backdropVertices);
-			glTexCoordPointer(2, GL_FLOAT, 0, backdropTexCoords);
-
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
-
-			// Then the sprite
-			glEnable(GL_BLEND);
-			glBindTexture (GL_TEXTURE_2D, fontPal.burnTex_names[single.texNum]);
-			glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); // GL_MODULATE instead of decal mixes the colours!
-
 			const GLfloat spriteVertices[] = { 
 				-xoffset, -yoffset, 0.0f, 
 				single.width-1-xoffset, -yoffset, 0.0f, 
-				single.width-1-xoffset, single.height-1-yoffset, 0.0f, 
-				-xoffset, single.height-1-yoffset, 0.0f
+				-xoffset, single.height-1-yoffset, 0.0f,
+				single.width-1-xoffset, single.height-1-yoffset, 0.0f
 			};
 
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+			glBindTexture (GL_TEXTURE_2D, backdropTextureName);
+			glUseProgram(shader.texture);
+			setPMVMatrix(shader.texture);
 
-			glVertexPointer(3, GL_FLOAT, 0, spriteVertices);
-			glTexCoordPointer(2, GL_FLOAT, 0, spriteTexCoords);
+			drawQuad(shader.texture, backdropVertices, 1, backdropTexCoords2);
 
-			glDrawArrays(GL_QUADS, 0, 4);
+			glActiveTexture(GL_TEXTURE2);
+			glBindTexture (GL_TEXTURE_2D, backdropTextureName);
+			glActiveTexture(GL_TEXTURE0);
 
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
+			glUseProgram(shader.paste);
+			GLint uniform = glGetUniformLocation(shader.paste, "useLightTexture");
+			if (uniform >= 0) glUniform1i(uniform, 0); // No lighting
 
-			glDisable(GL_BLEND);
+			setPMVMatrix(shader.paste);
+
+			glBindTexture (GL_TEXTURE_2D, fontPal.burnTex_names[single.texNum]);
+
+//FIXME: Test this some more. Also pasting the backdrop again is not strictly necessary but allows using the paste shader.
+			drawQuad(shader.paste, spriteVertices, 3, spriteTexCoords, NULL,  backdropTexCoords);
+
+			glUseProgram(0);
 
 			// Copy Our ViewPort To The Texture
-			glBindTexture(GL_TEXTURE_2D, backdropTextureName);
-			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, (x1<0) ? xoffset : x1+xoffset, (y1<0) ? yoffset: y1+yoffset, viewportOffsetX, viewportOffsetY, w, h);
+			copyTexSubImage2D(GL_TEXTURE_2D, 0, (x1<0) ? xoffset : x1+xoffset, (y1<0) ? yoffset: y1+yoffset, viewportOffsetX, viewportOffsetY, w, h, backdropTextureName);
 
 			yoffset += viewportHeight;
 		}
@@ -566,11 +606,11 @@ void burnSpriteToBackDrop (int x1, int y1, sprite & single, const spritePalette
 
 extern GLuint backdropTextureName;
 
-void fontSprite (int x, int y, sprite & single, const spritePalette & fontPal) {
+void fontSprite (bool flip, int x, int y, sprite & single, const spritePalette & fontPal) {
 
 	float tx1 = (float)(single.tex_x-0.5) / fontPal.tex_w[single.texNum];
 	float ty1 = 0.0;
-	float tx2 = (float)(single.tex_x + single.width+0.5) / fontPal.tex_w[single.texNum];
+	float tx2 = (float)(single.tex_x + single.width+(flip ? 1.0 : 0.5)) / fontPal.tex_w[single.texNum];
 	float ty2 = (float)(single.height+2)/fontPal.tex_h[single.texNum];
 
 	float x1 = (float)x - (float)single.xhot/cameraZoom;
@@ -578,99 +618,56 @@ void fontSprite (int x, int y, sprite & single, const spritePalette & fontPal) {
 	float x2 = x1 + (float)single.width/cameraZoom;
 	float y2 = y1 + (float)single.height/cameraZoom;
 
-	const GLfloat vertices[] = { 
+	GLfloat vertices[] = { 
 		x1, y1, 0.0f, 
 		x2, y1, 0.0f, 
-		x2, y2, 0.0f, 
-		x1, y2, 0.0f
+		x1, y2, 0.0f,
+		x2, y2, 0.0f
 	};
+	if (flip) {
+		vertices[0] = x2;
+		vertices[3] = x1;
+		vertices[6] = x2;
+		vertices[9] = x1;
+	}
 
 	const GLfloat texCoords[] = { 
 		tx1, ty1,
 		tx2, ty1,
-		tx2, ty2, 
-		tx1, ty2
+		tx1, ty2,
+		tx2, ty2
 	}; 
 
-	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); // GL_MODULATE instead of decal mixes the colours!
-	glColor3ub (fontPal.originalRed, fontPal.originalGreen, fontPal.originalBlue);
+	//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); // GL_MODULATE instead of decal mixes the colours!
+	setPrimaryColor(fontPal.originalRed/255.f, fontPal.originalGreen/255.f, fontPal.originalBlue/255.f, 1.0f);
 	glBindTexture (GL_TEXTURE_2D, fontPal.tex_names[single.texNum]);
 
+	glUseProgram(shader.smartScaler);
+	GLuint uniform = glGetUniformLocation(shader.smartScaler, "useLightTexture");
+	if (uniform >= 0) glUniform1i(uniform, 0);
+
+	setPMVMatrix(shader.smartScaler);
+
 	if (gameSettings.antiAlias == 1) {
-		glUseProgram(shader.smartScaler);
-		GLuint uniform = glGetUniformLocation(shader.smartScaler, "useLightTexture");
-		if (uniform >= 0) glUniform1i(uniform, 0);
+		glUniform1i(glGetUniformLocation(shader.smartScaler, "antialias"), 1);
+	} else {
+		glUniform1i(glGetUniformLocation(shader.smartScaler, "antialias"), 0);
 	}
 
 	glEnable(GL_BLEND);
 
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-	glVertexPointer(3, GL_FLOAT, 0, vertices);
-	glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
-
-	glDrawArrays(GL_QUADS, 0, 4);
-
-	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-	glDisableClientState(GL_VERTEX_ARRAY);
+	drawQuad(shader.smartScaler, vertices, 1, texCoords);
 
 	glDisable(GL_BLEND);
 	glUseProgram(0);
 }
 
+void fontSprite (int x, int y, sprite & single, const spritePalette & fontPal) {
+	fontSprite (false, x, y, single, fontPal);
+}
+
 void flipFontSprite (int x, int y, sprite & single, const spritePalette & fontPal) {
-
-	float tx1 = (float)(single.tex_x-0.5) / fontPal.tex_w[single.texNum];
-	float ty1 = 0.0;
-	float tx2 = (float)(single.tex_x + single.width+1.0) / fontPal.tex_w[single.texNum];
-	float ty2 = (float)(single.height+2)/fontPal.tex_h[single.texNum];
-
-	float x1 = (float)x - (float)single.xhot/cameraZoom;
-	float y1 = (float)y - (float)single.yhot/cameraZoom;
-	float x2 = x1 + (float)single.width/cameraZoom;
-	float y2 = y1 + (float)single.height/cameraZoom;
-
-	const GLfloat vertices[] = { 
-		x2, y1, 0.0f, 
-		x1, y1, 0.0f, 
-		x1, y2, 0.0f, 
-		x2, y2, 0.0f
-	};
-
-	const GLfloat texCoords[] = { 
-		tx1, ty1,
-		tx2, ty1,
-		tx2, ty2, 
-		tx1, ty2
-	}; 
-
-	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); // GL_MODULATE instead of decal mixes the colours!
-	glColor3ub (fontPal.originalRed, fontPal.originalGreen, fontPal.originalBlue);
-	glBindTexture (GL_TEXTURE_2D, fontPal.tex_names[single.texNum]);
-
-	if (gameSettings.antiAlias == 1) {
-		glUseProgram(shader.smartScaler);
-		GLuint uniform = glGetUniformLocation(shader.smartScaler, "useLightTexture");
-		if (uniform >= 0) glUniform1i(uniform, 0);
-	}
-
-	glEnable(GL_BLEND);
-
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-	glVertexPointer(3, GL_FLOAT, 0, vertices);
-	glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
-
-	glDrawArrays(GL_QUADS, 0, 4);
-
-	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-	glDisableClientState(GL_VERTEX_ARRAY);
-
-	glDisable(GL_BLEND);
-	glUseProgram(0);
-
+	fontSprite (true, x, y, single, fontPal);
 }
 
 
@@ -680,12 +677,12 @@ unsigned char curLight[3];
 
 void setDrawMode (onScreenPerson * thisPerson) {
 	if (thisPerson->colourmix) {
-		glEnable(GL_COLOR_SUM);
-		glSecondaryColor3ub (curLight[0]*thisPerson->r*thisPerson->colourmix/65025, curLight[1]*thisPerson->g*thisPerson->colourmix/65025, curLight[2]*thisPerson->b*thisPerson->colourmix/65025);
+		//glEnable(GL_COLOR_SUM); FIXME: replace line?
+		setSecondaryColor(curLight[0]*thisPerson->r*thisPerson->colourmix/65025/255.f, curLight[1]*thisPerson->g*thisPerson->colourmix/65025/255.f, curLight[2]*thisPerson->b*thisPerson->colourmix/65025/255.f, 1.0f);
 	}
 
-	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-	glColor4ub (curLight[0]*(255-thisPerson->colourmix)/255, curLight[1]*(255-thisPerson->colourmix)/255, curLight[2]*(255-thisPerson->colourmix)/255, 255 - thisPerson->transparency);
+	//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+	setPrimaryColor(curLight[0]*(255-thisPerson->colourmix)/65025.f, curLight[1]*(255-thisPerson->colourmix)/65025.f, curLight[2]*(255-thisPerson->colourmix)/65025.f, 1.0f - thisPerson->transparency/255.f);
 }
 
 extern GLuint backdropTextureName;
@@ -764,8 +761,8 @@ bool scaleSprite (sprite & single, const spritePalette & fontPal, onScreenPerson
 	const GLfloat ltexCoords[] = { 
 		ltx1, lty1,
 		ltx2, lty1,
-		ltx2, lty2, 
-		ltx1, lty2
+		ltx1, lty2,
+		ltx2, lty2
 	}; 
 
 	if (light && lightMap.data) {
@@ -789,13 +786,8 @@ bool scaleSprite (sprite & single, const spritePalette & fontPal, onScreenPerson
 			curLight[2] = target[2];
 		} else if (lightMapMode == LIGHTMAPMODE_PIXEL) {
 			curLight[0] = curLight[1] = curLight[2] = 255;
-			glClientActiveTexture(GL_TEXTURE1);
 			glActiveTexture(GL_TEXTURE1);
-			glEnable(GL_TEXTURE_2D);
 			glBindTexture (GL_TEXTURE_2D, lightMap.name);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-			glTexCoordPointer(2, GL_FLOAT, 0, ltexCoords);
-			glClientActiveTexture(GL_TEXTURE0);
 			glActiveTexture(GL_TEXTURE0);
 		}
 	} else {
@@ -811,17 +803,23 @@ bool scaleSprite (sprite & single, const spritePalette & fontPal, onScreenPerson
 
 	glEnable(GL_BLEND);
 
+	glUseProgram(shader.smartScaler);
+	GLuint uniform = glGetUniformLocation(shader.smartScaler, "useLightTexture");
+	if (uniform >= 0) glUniform1i(uniform, light && lightMapMode == LIGHTMAPMODE_PIXEL && lightMap.data);
+
+	setPMVMatrix(shader.smartScaler);
+
 	if (gameSettings.antiAlias == 1) {
-		glUseProgram(shader.smartScaler);
-		GLuint uniform = glGetUniformLocation(shader.smartScaler, "useLightTexture");
-		if (uniform >= 0) glUniform1i(uniform, light && lightMapMode == LIGHTMAPMODE_PIXEL && lightMap.data);
+		glUniform1i(glGetUniformLocation(shader.smartScaler, "antialias"), 1);
+	} else {
+		glUniform1i(glGetUniformLocation(shader.smartScaler, "antialias"), 0);
 	}
 
 	const GLfloat vertices[] = { 
 		x1, y1, z, 
 		x2, y1, z, 
-		x2, y2, z, 
-		x1, y2, z
+		x1, y2, z,
+		x2, y2, z
 	};
 
 	if (! mirror) {
@@ -832,34 +830,21 @@ bool scaleSprite (sprite & single, const spritePalette & fontPal, onScreenPerson
 	const GLfloat texCoords[] = { 
 		tx2, ty1,
 		tx1, ty1,
-		tx1, ty2, 
-		tx2, ty2
+		tx2, ty2,
+		tx1, ty2
 	}; 
 
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-	glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
-	glVertexPointer(3, GL_FLOAT, 0, vertices);
-
-	glDrawArrays(GL_QUADS, 0, 4);
-
-	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-	glDisableClientState(GL_VERTEX_ARRAY);
+	drawQuad(shader.smartScaler, vertices, 2, texCoords, ltexCoords);
 
 	glDisable(GL_BLEND);
 	glUseProgram(0);
 
 	if (light && lightMapMode == LIGHTMAPMODE_PIXEL) {
-		glClientActiveTexture(GL_TEXTURE1);
 		glActiveTexture(GL_TEXTURE1);
-		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-		glDisable(GL_TEXTURE_2D);
-		glClientActiveTexture(GL_TEXTURE0);
 		glActiveTexture(GL_TEXTURE0);
 	}
-	glSecondaryColor3ub (0, 0, 0);
-	glDisable(GL_COLOR_SUM);
+	setSecondaryColor(0., 0., 0.,1.);
+	//glDisable(GL_COLOR_SUM); FIXME: replace line?
 
 	// Are we pointing at the sprite?
 	if (input.mouseX >= x1 && input.mouseX <= x2 && input.mouseY >= y1 && input.mouseY <= y2) {
@@ -940,15 +925,15 @@ void fixScaleSprite (int x, int y, sprite & single, const spritePalette & fontPa
 	const GLfloat ltexCoords[] = { 
 		ltx1, lty1,
 		ltx2, lty1,
-		ltx2, lty2, 
-		ltx1, lty2
+		ltx1, lty2,
+		ltx2, lty2
 	}; 
 
 	const GLfloat btexCoords[] = { 
 		btx1, bty1,
 		btx2, bty1,
-		btx2, bty2, 
-		btx1, bty2
+		btx1, bty2,
+		btx2, bty2
 	}; 
 
 	if (light && lightMap.data) {
@@ -965,21 +950,14 @@ void fixScaleSprite (int x, int y, sprite & single, const spritePalette & fontPa
 			}
 		} else if (lightMapMode == LIGHTMAPMODE_PIXEL) {
 			curLight[0] = curLight[1] = curLight[2] = 255;
-			glClientActiveTexture(GL_TEXTURE1);
 			glActiveTexture(GL_TEXTURE1);
 			glBindTexture (GL_TEXTURE_2D, lightMap.name);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-			glTexCoordPointer(2, GL_FLOAT, 0, ltexCoords);
 		}
 	} else {
 		curLight[0] = curLight[1] = curLight[2] = 255;
 	}
-	glClientActiveTexture(GL_TEXTURE2);
 	glActiveTexture(GL_TEXTURE2);
 	glBindTexture (GL_TEXTURE_2D, backdropTextureName);
-	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-	glTexCoordPointer(2, GL_FLOAT, 0, btexCoords);
-	glClientActiveTexture(GL_TEXTURE0);
 	glActiveTexture(GL_TEXTURE0);
 
 	setPixelCoords (true);
@@ -993,7 +971,7 @@ void fixScaleSprite (int x, int y, sprite & single, const spritePalette & fontPa
 			int h = (diffY-yoffset< viewportHeight) ? (int) (diffY-yoffset) : viewportHeight;
 
 			// Render the scene - first the old backdrop (so that it'll show through when the z-buffer is active
-			glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+			//glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 			glBindTexture (GL_TEXTURE_2D, backdropTextureName);
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
@@ -1001,27 +979,21 @@ void fixScaleSprite (int x, int y, sprite & single, const spritePalette & fontPa
 			const GLfloat vertices[] = { 
 				-x1-xoffset, -y1-yoffset, 0.0f, 
 				sceneWidth-x1-xoffset, -y1-yoffset, 0.0f, 
-				sceneWidth-x1-xoffset, sceneHeight-y1-yoffset, 0.0f, 
-				-x1-xoffset, sceneHeight-y1-yoffset, 0.0f
+				-x1-xoffset, sceneHeight-y1-yoffset, 0.0f,
+				sceneWidth-x1-xoffset, sceneHeight-y1-yoffset, 0.0f
 			};
 
 			const GLfloat texCoords[] = { 
 				0.0f, 0.0f,
 				backdropTexW, 0.0f,
-				backdropTexW, backdropTexH, 
-				0.0f, backdropTexH
+				0.0f, backdropTexH,
+				backdropTexW, backdropTexH
 			}; 
-	
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 
-			glVertexPointer(3, GL_FLOAT, 0, vertices);
-			glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
+			glUseProgram(shader.texture);
+			setPMVMatrix(shader.texture);
 
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
+			drawQuad(shader.texture, vertices, 1, texCoords);
 
 			// The z-buffer
 			if (useZB) {
@@ -1038,6 +1010,8 @@ void fixScaleSprite (int x, int y, sprite & single, const spritePalette & fontPa
 			GLint uniform = glGetUniformLocation(shader.paste, "useLightTexture");
 			if (uniform >= 0) glUniform1i(uniform, light && lightMapMode == LIGHTMAPMODE_PIXEL && lightMap.data);
 
+			setPMVMatrix(shader.paste);
+
 			setDrawMode (thisPerson);
 
 			glBindTexture (GL_TEXTURE_2D, fontPal.tex_names[single.texNum]);
@@ -1045,8 +1019,8 @@ void fixScaleSprite (int x, int y, sprite & single, const spritePalette & fontPa
 			const GLfloat vertices2[] = { 
 				-xoffset, -yoffset, z, 
 				spriteWidth-xoffset, -yoffset, z, 
-				spriteWidth-xoffset, spriteHeight-yoffset, z, 
-				-xoffset, spriteHeight-yoffset, z
+				-xoffset, spriteHeight-yoffset, z,
+				spriteWidth-xoffset, spriteHeight-yoffset, z
 			};
 
 			if (! mirror) {
@@ -1057,40 +1031,19 @@ void fixScaleSprite (int x, int y, sprite & single, const spritePalette & fontPa
 			const GLfloat texCoords2[] = { 
 				tx2, ty1,
 				tx1, ty1,
-				tx1, ty2, 
-				tx2, ty2
+				tx2, ty2,
+				tx1, ty2
 			}; 
 
-			glEnableClientState(GL_VERTEX_ARRAY);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+			drawQuad(shader.paste, vertices2, 3, texCoords2, ltexCoords, btexCoords);
 
-			glVertexPointer(3, GL_FLOAT, 0, vertices2);
-			glTexCoordPointer(2, GL_FLOAT, 0, texCoords2);
-
-			glDrawArrays(GL_QUADS, 0, 4);
-
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glDisableClientState(GL_VERTEX_ARRAY);
-
-			glSecondaryColor3ub (0, 0, 0);
-			glDisable(GL_COLOR_SUM);
+			setSecondaryColor(0., 0., 0., 1.);
+			//glDisable(GL_COLOR_SUM); FIXME: replace line?
 
 			// Copy Our ViewPort To The Texture
-			glBindTexture(GL_TEXTURE_2D, backdropTextureName);
 			glUseProgram(0);
 
-			if (light && lightMapMode == LIGHTMAPMODE_PIXEL) {
-				glClientActiveTexture(GL_TEXTURE1);
-				glActiveTexture(GL_TEXTURE1);
-				glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			}
-			glClientActiveTexture(GL_TEXTURE2);
-			glActiveTexture(GL_TEXTURE2);
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-			glClientActiveTexture(GL_TEXTURE0);
-			glActiveTexture(GL_TEXTURE0);
-
-			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, (int) ((x1<0) ? xoffset: x1+xoffset), (int) ((y1<0) ? yoffset: y1+yoffset), (int) ((x1<0) ?viewportOffsetX-x1:viewportOffsetX), (int) ((y1<0) ?viewportOffsetY-y1:viewportOffsetY), w, h);
+			copyTexSubImage2D(GL_TEXTURE_2D, 0, (int) ((x1<0) ? xoffset: x1+xoffset), (int) ((y1<0) ? yoffset: y1+yoffset), (int) ((x1<0) ?viewportOffsetX-x1:viewportOffsetX), (int) ((y1<0) ?viewportOffsetY-y1:viewportOffsetY), w, h, backdropTextureName);
 
 			yoffset += viewportHeight;
 		}
