@@ -28,9 +28,9 @@ bool saveThumbnail (FILE * fp) {
 		if (! freeze ()) return false;
 
 
-		glEnable (GL_TEXTURE_2D);
 		setPixelCoords (true);
-		glUseProgram(0);
+		glUseProgram(shader.texture);
+		setPMVMatrix(shader.texture);
 
 		glGenTextures (1, &thumbnailTextureName);
 		glBindTexture(GL_TEXTURE_2D, thumbnailTextureName);
@@ -38,38 +38,29 @@ bool saveThumbnail (FILE * fp) {
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-		glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, thumbWidth, thumbHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
+		texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, thumbWidth, thumbHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0, thumbnailTextureName);
 
 		// Render the backdrop (scaled)
-		glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+		//glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 		glBindTexture (GL_TEXTURE_2D, backdropTextureName);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 
-		const GLint vertices[] = { 
-			0, 0, 0, 
-			thumbWidth-1, 0, 0, 
-			thumbWidth-1, thumbHeight-1, 0, 
-			0, thumbHeight-1, 0
+		const GLfloat vertices[] = { 
+			0., 0., 0., 
+			thumbWidth-1.f, 0., 0., 
+			0., thumbHeight-1.f, 0.,
+			thumbWidth-1.f, thumbHeight-1.f, 0.
 		};
 
 		const GLfloat texCoords[] = { 
 			0.0f, 0.0f,
 			backdropTexW, 0.0f,
-			backdropTexW, backdropTexH, 
-			0.0f, backdropTexH
+			0.0f, backdropTexH,
+			backdropTexW, backdropTexH
 		}; 
 
-		glEnableClientState(GL_VERTEX_ARRAY);
-		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-		glVertexPointer(3, GL_INT, 0, vertices);
-		glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
-
-		glDrawArrays(GL_QUADS, 0, 4);
-
-		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-		glDisableClientState(GL_VERTEX_ARRAY);
+		drawQuad(shader.texture, vertices, 1, texCoords);
 
 		if (gameSettings.antiAlias < 0) {
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
@@ -78,13 +69,15 @@ bool saveThumbnail (FILE * fp) {
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 		}
-		glDeleteTextures (1, &thumbnailTextureName);
+		deleteTextures (1, &thumbnailTextureName);
 		thumbnailTextureName = 0;
 		
 		// Save Our ViewPort
 		GLushort* image = new GLushort [thumbWidth*thumbHeight];
+		if (! checkNew (image)) return false;
 		glReadPixels(viewportOffsetX, viewportOffsetY, thumbWidth, thumbHeight, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, image);
 
+		glUseProgram(0);
 		setPixelCoords (false);
 
 		for (int y = 0; y < thumbHeight; y ++) {
@@ -104,6 +97,9 @@ void showThumbnail (char * filename, int atX, int atY) {
 	GLubyte * thumbnailTexture = NULL;
 	GLuint thumbnailTextureName = 0;
 
+	GLfloat texCoordW = 1.0;
+	GLfloat texCoordH = 1.0;	
+	
 	int ssgVersion;
 	FILE * fp = openAndVerify (filename, 'S', 'A', ERROR_GAME_LOAD_NO, ssgVersion);
 	if (ssgVersion >= VERSION(1,4)) {
@@ -116,6 +112,9 @@ void showThumbnail (char * filename, int atX, int atY) {
 		if (! NPOT_textures) {
 			picWidth = getNextPOT(picWidth);
 			picHeight = getNextPOT(picHeight);
+			texCoordW = ((double)fileWidth) / picWidth;
+			texCoordH = ((double)fileHeight) / picHeight;
+			
 		}
 
 		thumbnailTexture = new GLubyte [picHeight*picWidth*4];
@@ -145,7 +144,7 @@ void showThumbnail (char * filename, int atX, int atY) {
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-		glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, thumbnailTexture);
+		texImage2D (GL_TEXTURE_2D, 0, GL_RGBA, picWidth, picHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, thumbnailTexture, thumbnailTextureName);
 
 		delete thumbnailTexture;
 		thumbnailTexture = NULL;
@@ -161,8 +160,11 @@ void showThumbnail (char * filename, int atX, int atY) {
 		if (fileWidth + atX > sceneWidth) fileWidth = sceneWidth-atX;
 		if (fileHeight + atY > sceneHeight) fileHeight = sceneHeight-atY;
 
-		glEnable (GL_TEXTURE_2D);
 		setPixelCoords (true);
+
+		glUseProgram(shader.texture);
+		setPMVMatrix(shader.texture);
+
 		int xoffset = 0;
 		while (xoffset < fileWidth) {
 			int w = (fileWidth-xoffset < viewportWidth) ? fileWidth-xoffset : viewportWidth;
@@ -173,44 +175,34 @@ void showThumbnail (char * filename, int atX, int atY) {
 
 				glBindTexture (GL_TEXTURE_2D, thumbnailTextureName);
 
-				const GLint vertices[] = { 
-					fileWidth-1-xoffset, -yoffset, 0, 
-					-xoffset, -yoffset, 0, 
-					-xoffset, fileHeight-1-yoffset, 0, 
-					fileWidth-1-xoffset, fileHeight-1-yoffset, 0
+				const GLfloat vertices[] = { 
+					(GLfloat)fileWidth-1.f-xoffset, (GLfloat)-yoffset, 0., 
+					(GLfloat)-xoffset, (GLfloat)-yoffset, 0., 
+					(GLfloat)fileWidth-1.f-xoffset, (GLfloat)fileHeight-1.f-yoffset, 0.,
+					(GLfloat)-xoffset, (GLfloat)fileHeight-1.f-yoffset, 0.
 				};
 
 				const GLfloat texCoords[] = { 
-					backdropTexW, 0.0f,
+					texCoordW, 0.0f,
 					0.0f, 0.0f,
-					0.0f, backdropTexH, 
-					backdropTexW, backdropTexH
+					texCoordW, texCoordH,
+					0.0f, texCoordH
 				}; 
 	
-				glEnableClientState(GL_VERTEX_ARRAY);
-				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-				glVertexPointer(3, GL_INT, 0, vertices);
-				glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
-
-				glDrawArrays(GL_QUADS, 0, 4);
-
-				glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-				glDisableClientState(GL_VERTEX_ARRAY);
+				drawQuad(shader.texture, vertices, 1, texCoords);
 
 				glDisable(GL_BLEND);
 
 				// Copy Our ViewPort To The Texture
-				glBindTexture(GL_TEXTURE_2D, backdropTextureName);
-				glCopyTexSubImage2D(GL_TEXTURE_2D, 0, atX+xoffset, atY+yoffset, viewportOffsetX, viewportOffsetY, w, h);
+				copyTexSubImage2D(GL_TEXTURE_2D, 0, atX+xoffset, atY+yoffset, viewportOffsetX, viewportOffsetY, w, h, backdropTextureName);
 
 				yoffset += viewportHeight;
 			}
 			xoffset += viewportWidth;
 		}
-
+		glUseProgram(0);
 		setPixelCoords (false);
-		glDeleteTextures (1, &thumbnailTextureName);
+		deleteTextures (1, &thumbnailTextureName);
 		thumbnailTextureName = 0;
 	}
 }
